// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js


(function (define, angular) {
  'use strict';

  define([
    'config',
    'appRouter',
    'services/appInit',

    //'data/websocketData',
    'data/mowsTransport',
    //'data/pahoTransport',
    'data/transportBroker',

    'data/BaseData',
    'data/collectionData',
    'data/userProfile',
    'data/listData',
    'data/chatsData',
    'data/chatsCollection',
    'data/chatsArchivedData',
    'data/messageCollection',
    'data/matchesData',
    'data/imageData',
    'data/contactList',
    'data/pushNotificationData',
    'data/settingsData',

    'directives/kmImgDir',
    'directives/kmImgBgDir',
    'directives/kmMatchImgDir',
    'directives/kmBadgeDir',
    'directives/kmGlobalErrorDir',
    'directives/kmOfflineDisableDir',
    'directives/kmMakeSquareDir',
    'directives/kmStayFocusedDir',
    'directives/kmInfiniteScrollDir',
    'directives/kmRefresherDir',
    'directives/kmUndoDir',

    'chats/ChatsCtrl',
    'dialogs/dialogs',
    'dialogs/uploadImageDialog',
    'dialogs/chatPlusDialog',
    'dialogs/fullScreenDialog',
    'dialogs/cropImageDialog',
    'dialogs/notificationPopover',
    'dialogs/newMatchDialog',

    'services/appEvents',
    'services/appState',
    'services/localNotification',
    'services/security',
    'services/touchAuth',
    'services/localeLang',
    'services/geoLocation',
    'services/emailAvailable',

    // separate modules (no need to include in function):
    'controllers',
    'chats/ChatsModule',
    'matches/MatchesModule',
    'media/MediaModule'
  ], function(config, appRouter, appInit,
    mqttTransport, transportBroker,
    baseData, collectionData, userProfile, listData, chatsData, chatsCollection, chatsArchivedData, messageCollection, matchesData, imageData, contactList, pushData, settingsData,
    kmImgDir, kmImgBgDir, kmMatchImgDir, kmBadgeDir, kmGlobalErrorDir, kmOfflineDisableDir, kmMakeSquareDir, kmStayFocusedDir, kmInfiniteScrollDir, kmRefresherDir, kmUndoDir,
    ChatsCtrl, dialogs, uploadImageDialog, chatPlusDialog, fullScreenDialog, cropImageDialog, notificationPopover, newMatchDialog,
    appEvents, appState, localNotification, security, touchAuth, localeLang, geoLocation, emailAvailable
  ) {

    var app = angular.module('Kismet', [
      'ionic', 'angularMoment', 'uuid4', 'monospaced.elastic', 'ngImgCrop', 'pascalprecht.translate',
      'Kismet.controllers', 'Kismet.chats', 'Kismet.matches', 'Kismet.media'
    ]);

    app.run(function($location, appInit, userProfile){
      appInit.onStart();

      if (userProfile.isLoggedIn()){
        userProfile
          .getLoginInfo()
          .then(function(loginInfo){
            $rootScope.$broadcast('logged-in', loginInfo);
            if (loginInfo.profileComplete) {
              $location.path('/app/matches');
              userProfile.loadInfo(false).then(function(data){
                $rootScope.$broadcast('profile-ready', data);
                // revalidate on background:
                //userProfile.loadInfo(true).then(function(data){
                //  var profile = userProfile.getFull(),
                //    imageId = profile.imageId;
                //  $rootScope.$broadcast('profile-image-updated', {imageId: imageId});
                //  //$rootScope.$broadcast('profile-ready', profile);
                //});
              }).catch(function(error){
                console.log('[app.run] ERROR: could not load user profile: ' + error, error);
                $location.path('/app/force-logout');
              });
            } else {
              $location.path('/app/force-logout');
            }
          });
      } else {
        $location.path('/start/first-run');
      }
    });

    app.filter('reverse', function() {
      return function(items) {
        return items.slice().reverse();
      };
    });

    app.filter('contactFormattedName', function() {
      return function(items) {
        return items.sort(function(a, b){
          if(a.name.formatted < b.name.formatted) return -1;
          if(a.name.formatted > b.name.formatted) return 1;
          return 0;
        });
      };
    });

    app.factory('appInit', appInit);
    //app.factory('websocketData', websocketData);
    app.factory('mqttTransport', mqttTransport);
    app.factory('transportBroker', transportBroker);

    app.factory('baseData', baseData);
    app.factory('collectionData', collectionData);
    app.factory('userProfile', userProfile);
    app.factory('listData', listData);
    app.factory('chatsData', chatsData);
    app.factory('chatsCollection', chatsCollection);
    app.factory('chatsArchivedData', chatsArchivedData);
    app.factory('messageCollection', messageCollection);
    app.factory('matchesData', matchesData);
    app.factory('imageData', imageData);
    app.factory('contactListData', contactList);
    app.factory('pushData', pushData);
    app.factory('settingsData', settingsData);

    app.factory('appEvents', appEvents);
    app.factory('appState', appState);
    app.factory('localNotification', localNotification);
    app.service('security', security);

    app.factory('dialogs', dialogs);
    app.factory('uploadImageDialog', uploadImageDialog);
    app.factory('chatPlusDialog', chatPlusDialog);
    app.factory('fullScreenDialog', fullScreenDialog);
    app.factory('cropImageDialog', cropImageDialog);
    app.factory('notificationPopover', notificationPopover);
    app.factory('newMatchDialog', newMatchDialog);

    app.factory('touchAuth', touchAuth);
    app.factory('localeLang', localeLang);
    app.factory('geoLocation', geoLocation);
    app.factory('emailAvailable', emailAvailable);

    app.directive('kmImg', kmImgDir);
    app.directive('kmImgBg', kmImgBgDir);
    app.directive('kmMatchImg', kmMatchImgDir);
    app.directive('kmBadge', kmBadgeDir);
    app.directive('kmGlobalError', kmGlobalErrorDir);
    app.directive('kmOfflineDisable', kmOfflineDisableDir);
    app.directive('kmMakeSquare', kmMakeSquareDir);
    app.directive('kmStayFocused', kmStayFocusedDir);
    app.directive('kmInfiniteScroll', kmInfiniteScrollDir);
    app.directive('kmRefresher', kmRefresherDir);
    app.directive('kmUndo', kmUndoDir);

    app.config(function ($stateProvider, $urlRouterProvider, $translateProvider) {
      $translateProvider.useStaticFilesLoader({
        prefix: 'languages/',
        suffix: '.json'
      });
      $translateProvider.preferredLanguage(config.lang.defaultLocale);
    });

    app.config(appRouter);

    return app;
  });

}(define, angular));
