/**
 * Dialog fullscreen (e.g. for full image view)
 */

(function (define) {
  'use strict';

  define(['config', 'chats/ChatsCtrl'], function (config, ChatsCtrl) {

    var appRouter = function ($stateProvider, $urlRouterProvider) {

        $stateProvider
          .state('start', {
            url: "/start",
            abstract: true,
            templateUrl: "templates/start.html",
            controller: 'AppCtrl'
          })
          .state('start.first-run', {
            url: "/first-run",
            views: {
              'start': {
                templateUrl: "templates/first-run.html",
                controller: function($scope) {
                  $scope.setPage('first-run');
                  if (window.StatusBar) StatusBar.styleDefault();
                  $scope.clearErrors();
                  $scope.clearLoginInfo();
                }
              }
            }
          })
          .state('start.login', {
            url: "/login",
            views: {
              'start': {
                templateUrl: "templates/login.html",
                controller: function($scope){
                  $scope.setPage('login');
                  if (window.StatusBar) StatusBar.styleLightContent();
                }
              }
            }
          })
          .state('start.signup', {
            url: "/signup",
            views: {
              'start': {
                templateUrl: "templates/signup.html",
                controller: function($scope, userProfile){
                  $scope.setPage('singup');
                  if (window.StatusBar) StatusBar.styleLightContent();
                  if (userProfile.isLoggedIn()) $scope.logout();
                }
              }
            }
          })
          .state('start.verify', {
            url: "/verify",
            views: {
              'start': {
                templateUrl: "templates/verify.html",
                controller: function($scope){
                  $scope.setPage('verify');
                  if (window.StatusBar) StatusBar.styleLightContent();
                }
              }
            }
          })
          .state('start.empty-profile', {
            url: "/empty-profile",
            views: {
              'start': {
                templateUrl: "templates/edit-profile.html",
                controller: "EditProfileCtrl"
              }
            }
          })
          .state('app.force-logout', {
            url: "/force-logout",
            views: {
              'menuContent': {
                templateUrl: "templates/blank.html",
                controller: function($scope, $location, userProfile){
                  //console.log('Page controller: force-logout');
                  userProfile.logout();
                  $scope.isLoggedIn = false;
                  $location.path('/start/first-run');
                }
              }
            }
          })
          .state('app', {
            url: "/app",
            abstract: true,
            templateUrl: "templates/menu.html",
            controller: 'AppCtrl'
          })
          .state('app.edit-profile', {
            url: "/edit-profile",
            views: {
              'menuContent': {
                templateUrl: "templates/edit-profile.html",
                controller: "EditProfileCtrl"
              }
            }
          })
          .state('app.my-profile', {
            url: "/my-profile",
            views: {
              'menuContent': {
                templateUrl: "templates/my-profile.html",
                controller: "EditProfileCtrl"
              }
            }
          })
          .state('app.profiles', {
            url: "/profiles/:profileId",
            views: {
              'menuContent': {
                templateUrl: "templates/profiles.html",
                controller: "ProfilesCtrl"
              }
            }
          })
          .state('app.matches', {
            url: "/matches",
            views: {
              'menuContent': {
                templateUrl: "templates/matches.html",
                controller: 'MatchesCtrl'
              }
            }
          })
          .state('app.change-pass', {
            url: "/change-pass",
            views: {
              'menuContent': {
                templateUrl: "templates/change-pass.html",
                controller: 'ChangePassCtrl'
              }
            }
          })
          .state('app.chats', {
            url: "/chats/:type",  // <active | archived>
            views: {
              'menuContent': {
                templateUrl: "templates/chats.html",
                controller: function ($scope, $stateParams, $q, chatsCollection, chatsArchivedData, userProfile, $ionicPopup, baseData, messageCollection, notificationPopover, $ionicPopover, $translate, dialogs) {
                  $scope.isArchive = $stateParams.type === 'archived';
                  $scope.isActive = !$scope.isArchive;
                  $scope.title = ($scope.isActive ? 'TITLE_ACTIVE_CHAT' : 'TITLE_ARCHIVED_CHAT');

                  ChatsCtrl($scope, $q, ($scope.isActive ? chatsCollection : chatsArchivedData), ($scope.isActive ? chatsArchivedData : chatsCollection), userProfile, $ionicPopup, baseData, messageCollection, notificationPopover, $ionicPopover, $translate, dialogs);
                }
              }
            }
          })
          .state('app.chatroom', {
            url: "/chat-room/:chatId",
            views: {
              'menuContent': {
                templateUrl: "templates/chat-room.html",
                controller: 'ChatRoomCtrl'
              }
            }
          })
          .state('app.connect', {
            url: "/connect",
            views: {
              'menuContent': {
                templateUrl: "templates/connect.html"
              }
            }
          })
          .state('app.generate', {
            url: "/connect/generate",
            views: {
              'menuContent': {
                templateUrl: "templates/generate.html",
                controller: "ConnectCtrl"
              }
            }
          })
          .state('app.enter', {
            url: "/connect/enter",
            views: {
              'menuContent': {
                templateUrl: "templates/enter.html",
                controller: "ConnectCtrl"
              }
            }
          })
          .state('app.settings', {
            url: "/settings",
            views: {
              'menuContent': {
                templateUrl: "templates/settings.html",
                controller: "SettingsCtrl"
              }
            }
          })
          .state('app.settings-advanced', {
            url: "/settings-advanced",
            views: {
              'menuContent': {
                templateUrl: "templates/settings-advanced.html",
                controller: "SettingsCtrl"
              }
            }
          })
          .state('app.notifications', {
            url: "/notifications",
            views: {
              'menuContent': {
                templateUrl: "templates/notifications.html",
                controller: "SettingsCtrl"
              }
            }
          })
          .state('app.contacts', {
            url: "/contacts",
            views: {
              'menuContent': {
                templateUrl: "templates/contacts.html",
                controller: "ContactsCtrl"
              }
            }
          })
          .state('app.terms', {
            url: "/terms",
            views: {
              'menuContent': {
                templateUrl: "templates/terms.html"
              }
            }
          })
          .state('app.aboutus', {
            url: "/aboutus",
            views: {
              'menuContent': {
                templateUrl: "templates/aboutus.html"
              }
            }
          });
        // if none of the above states are matched, use this as the fallback
        $urlRouterProvider.otherwise('/start/first-run');
      };

    return appRouter;
  })
}(define));
