(function (define) {
  'use strict';

  define(['config'], function (config) {

    // WARNING: this is used as a regular functin NOT as a controller with DI !!! (see appRouter.js)
    var ChatsCtrl = function ($scope, $q, chatsCollection, otherChatCollection, userProfile, $ionicPopup, baseData, messageCollection, notificationPopover, $ionicPopover, $translate, dialogs) {
      //console.log('ChatsCtrl: using chats collection');
      var myProfileId = userProfile.getProfileId();

      $scope.items = chatsCollection.take();
      $scope.isLoading = false;
      $scope.error = '';
      $scope.currentChatId = null;
      $scope.currentProfileId = null;
      $scope.onAndoid = false;
      if (window.device) {
        $scope.onAndoid = (device.platform == 'Android');
      }

      /***  Init steps  ***/
      chatsCollection.onUpdate($scope, function(items){
        //console.log('ChatsCtrl: collection onUpdate occur: ', items);
        $scope.items = items;
        $scope.isLoading = false;
      });
      if ($scope.items.length === 0 && !chatsCollection.wasLoaded()){
        $scope.isLoading = true;
        chatsCollection.load();
      }

      notificationPopover.listenAndPopup($scope, function(data){
        chatsCollection.updateChat({
          chatId: data.chatId,
          lastMessage: data.text,
          lastMessageFromYou: false,
          hasNewMessage: true
        });
        $scope.items = chatsCollection.take();


        //TODO: should we reload here?
        //chatsCollection.load();
      });

      $scope.$on('screenshot-taken', function(){
        var ids = chatsCollection._getAll().map(function(c){ return c.chatId;});
        for(var id in ids)
          {
            baseData.post('/api/chat/screen-capture?profileId={profileId}&chatId=' + ids[id], null, myProfileId);
          }
      });
      /***  View exposed methods  ***/
      $scope.wipeChat = function($event, chatId) {
        $scope.currentChatId = chatId;
        $event.preventDefault();
        $event.stopPropagation();
        navigator.vibrate(config.popups.confirmVibrateTime);
        $q.all({okText: $translate('POPUP_OK'), cancelText: $translate('POPUP_CANCEL'), title: $translate('POPUP_WIPE_TITLE'), text: $translate('POPUP_WIPE_TEXT')})
            .then(function(translation){
              var confirmPopup = $ionicPopup.confirm({
                title: translation.title,
                template: translation.text,
                okText: translation.okText,
                cancelText: translation.cancelText
              });
              confirmPopup.then(function(res) {
                if(res) {
                  wipeChat($scope.currentChatId);
                }
              });
            })
            .catch(function(){
              var confirmPopup = $ionicPopup.confirm({
                title: 'POPUP_WIPE_TITLE',
                template: 'POPUP_WIPE_TEXT',
                okText: 'POPUP_OK',
                cancelText: 'POPUP_CANCEL'
              });
              confirmPopup.then(function(res) {
                if(res) {
                  wipeChat($scope.currentChatId);
                }
              });
            });
      };
      $scope.archiveChat = function($event, chatId) {
        $event.preventDefault();
        $event.stopPropagation();
        archiveChat(chatId);
      };
      $scope.blockChat = function($event, chatId) {
        $event.preventDefault();
        $event.stopPropagation();
        blockUser(chatId);
      };

      $ionicPopover.fromTemplateUrl('templates/dialogs/chats-android-popover.html', {
        scope: $scope
      }).then(function(popover) {
        $scope.popover = popover;
        $scope.blockThisChat = function(){
          blockUser($scope.currentChatId).then(function(){
            $scope.popover.hide();
          });
        };
        $scope.archiveThisChat = function(){
          archiveChat($scope.currentChatId);
          $scope.popover.hide();
        };
        $scope.viewProfile = function() {
          $scope.goto('/app/profiles/' + $scope.currentProfileId);
        };
      });
      $scope.popoverOptions = function($event, chatId, profileId){
        $event.preventDefault();
        $event.stopPropagation();
        $scope.popover.show($event);
        $scope.currentChatId = chatId;
        $scope.currentProfileId = profileId;
      };
      $scope.$on('$destroy', function() {
        $scope.popover.remove();
      });
      $scope.$on('app-resume', function(){
        chatsCollection.clear();
        otherChatCollection.clear();
        chatsCollection.load();
      });

      /***  Chat manipulation functions  ***/
      function wipeChat(chatId) {
        baseData.post('/api/chat/delete-messages?profileId={profileId}&chatId=' + chatId, null, myProfileId);
        messageCollection(chatId).clear();
        archiveChat(chatId);
      }
      function blockUser(chatId) {
        return dialogs.blockUser().then(function(){
          return chatsCollection.blockUser(chatId, myProfileId);
        })
      }
      function archiveChat(chatId) {
        //console.log('Archiving chat: ' + chatId);
        baseData.post('/api/chat/archive?profileId={profileId}&chatId=' + chatId, null, myProfileId);

        var chat = chatsCollection.remove(chatId, true);
        if (chat) {
          //console.log('- chat was removed from active.', chat);
          otherChatCollection.add(chat);
          //console.log('- chat was added to archived.', chat);
        }
        otherChatCollection.clear();
      }
    };

    return ChatsCtrl;
  });

}(define));
