(function (env) {

  var envDependent = {
    test: {
      server: {
        host: 'api.test.discreetlyapp.com',
        proto: 'https',
        mqttProto: 'wss',
        mqttPort: 443
      }
    },
    prod: {
      server: {
        host: 'api.discreetlyapp.com',
        proto: 'https',
        mqttProto: 'wss',
        mqttPort: 443
      }
    },
    local: {
      server: {
        host: 'localhost',
        proto: 'http',
        port: 8000,
        mqttProto: 'ws',
        mqttPort: 8000
      }
    }
  };
  Object.keys(envDependent).forEach(function(key){
    var env = envDependent[key];
    env.server.url = env.server.proto + '://' + env.server.host + (env.server.port ? ':' + env.server.port : '');
    env.server.mqtt = env.server.mqttProto + '://' + env.server.host + (env.server.mqttPort ? ':' + env.server.mqttPort : '');
  });

  var config = {
    version: '1.0.0',
    defaultEnv: 'prod',
    lang: {
      languagesTitles: ['English', 'Français', 'Español'],
      localeCodes: ['en-ca', 'fr-ca', 'es'],
      defaultLocale: 'en-ca'
    },
    splash: {
      delay: 200
    },
    popups: {
      confirmVibrateTime: 500,
      alertVibrateTime: 500
    },
    security: {
      algorithm: 'sha256'
    },
    server: {},
    data: {
      matchesLoadPageSize: 25,
      matchesViewPageSize: 25
    },
    requestTtl: 10,
    ttl: {
      IMAGES: -1,
      CHATS: 30,
      CHATS_ARCHIVED: 30,
      MESSAGES: 30,
      FULL_IMAGE: 60*600,
      CHAT_TYPING_BUBBLE_SEC: 15,
      MATCH_UNDO_SEC: 1.5
    },
    push: {
      senderId: '778913537375'
    },
    popup: {
      ttl: 3
    },
    apiUrls: {
      LOGIN:          '/api/profile/login',
      SIGNUP:         '/api/profile/sign-up',
      PING:           '/api/ops/ping',
      LOAD_PROFILE:   '/api/profile/user-info?profileId={profileId}',
      SAVE_PROFILE:   '/api/profile/user-info?profileId={profileId}',
      REGISTER_DEVICE:'/api/profile/registration?profileId={profileId}',
      UPLOAD_MEDIA:   '/api/media?profileId={profileId}&chatId={chatId}&mediaType={mediaType}&mediaId={mediaId}',
      PLAY_MEDIA:     '/api/media?mediaId={mediaId}',
      IMAGE:          '/api/image?imageId={imageId}&size={size}&dataType={type}',
      MATCH_RECOMMEND:'/api/match/recommend?profileId={profileId}&firstBatch={firstBatch}&batchSize={batchSize}',
      MATCH_LIKE:     '/api/match/like?profileId={profileId}&likeProfileId={matchedId}&like={liked}',
      MATCH_REFRESH:  '/api/match/refresh?profileId={profileId}',
      MATCH_UNDO:     '/api/match/undo?profileId={profileId}&likeProfileId={matchedId}'
    },
    event: {
      NEW_MATCH: 'new-match',

      // chat events:
      CHAT_MESSAGE_NEW: 'chat-message-new',
      CHAT_MESSAGE_SENT: 'chat-message-sent',
      CHAT_MESSAGE_DELIVERED: 'chat-message-delivered',
      CHAT_MESSAGE_READ: 'chat-message-read',
      CHAT_MESSAGE_DELETED: 'chat-message-deleted',
      CHAT_MESSAGE_TYPING: 'chat-message-typing',
      CHAT_WIPE: 'chat-wipe',

      // scroll events:
      SCROLL_PTR_ACTIVATE: 'scroll-ptr-activate',
      SCROLL_PTR_DEACTIVATE: 'scroll-ptr-deactivate',
      SCROLL_PTR_START: 'scroll-ptr-start',

      // message transport events:
      TRANSPORT_SUB:          'transport-sub-msg',
      TRANSPORT_PUB:          'transport-pub-msg',
      TRANSPORT_PUB_ACK:      'transport-pub-ack',
      TRANSPORT_PUB_REJECTED: 'transport-pub-ack',

      SIGNIFICANT_CHANGE:     'significant-change'
    },
    geoLocationRefresh: {
      REFRESH_TIME: 600,
      REFRESH_ERROR: 3
    },
    media: {
      audioRecordLimit: 60,  // in seconds
      videoRecordLimit: 30
    },
    transport: {
      RECONNECT_PERIOD_MS: 1000,
      MQTT_KEEPALIVE_SEC: 60
    },
    termsUrl: 'http://discreetlyapp.com/terms'
  };

  config.getMediaUrl = function (mediaId) {
    return config.server.url + config.apiUrls.PLAY_MEDIA.replace('{mediaId}', mediaId);
  };
  config.setEnv = function(env){
    if (envDependent[env] && config.env !== env){
      Object.keys(envDependent[env]).forEach(function(prop){
        config[prop] = envDependent[env][prop];
      });
      localStorage.setItem('kismet-env', env);
      config.env = env;
    }
  };

  var availableEnvs = Object.keys(envDependent),
    //localEnv = localStorage.getItem('kismet-env') || config.defaultEnv;
    localEnv = config.defaultEnv;
  config.setEnv(availableEnvs.indexOf(localEnv) !== -1 ? localEnv : availableEnvs[0]);

  if (typeof exports === 'object') {
    module.exports = config;
  } else if (typeof define === 'function' && define.amd) {
    define(config);
  }
}());