(function (define) {
  'use strict';

  define(['config'], function (config) {

    var URL = config.apiUrls;

    var AppCtrl = function($scope, $rootScope, $q, $ionicPopup, $ionicActionSheet, $ionicSideMenuDelegate, $location, userProfile, $translate, emailAvailable, $ionicPlatform, baseData) {
      $scope.isLoggedIn = userProfile.isLoggedIn();
      $scope.threeFails = 0;
      $scope.onPage = 'first-run';
      console.log('AppCtrl ' + $scope.onPage);
      $scope.setPage = function(page){
        $scope.onPage = page;
      };

      $ionicPlatform.onHardwareBackButton(function() {
        if ($location.path().search('first-run')) {
          ionic.Platform.exitApp();
        }
      });

      // Form data for the login modal
      $scope.loginData = {pin:'', countryCode:'+1'};
      $scope.userInfo = {};
      $scope.profileImageP = userProfile.getImageId();
      $scope.profileImageControl = {};
      $scope.error = '';
      $scope.errors = {};
      $scope.menuSelected = 'menu-matches';

      init();
      $scope.$on('profile-ready', init);
      $scope.$on('profile-image-updated', function(){
        userProfile.getImageId().then(function(imageId){
          $scope.profileImageControl.refresh && $scope.profileImageControl.refresh(imageId);
        })
      });

      $scope.clearErrors = function() {
        $scope.error = '';
        $scope.errors = {};
      };

      $scope.clearLoginInfo = function() {
        $scope.loginData = {pin:'', countryCode:'+1'};
      };

      $scope.checkUsername = function() {
        if ($scope.loginData.username) {
          baseData.post('/api/profile/username-availability', {username: $scope.loginData.username})
            .then(function (data) {
              if (!data.available) {
                $scope.errors.email = 'FORM_ERROR_USER_ID_NOT_AVAILABLE';
              }
            });
        }
      };

      function init() {
        var userFullInfo = userProfile.getFull();
        if (!userFullInfo) {
          return;
        }
        $scope.error = '';
        $scope.errors = {};
        $scope.userInfo = userFullInfo;
        $scope.profileImage = userFullInfo.imageId;
      }

      $scope.changeFirstName = function(newName) {
        $scope.userInfo.firstName = newName;
      };

      $scope.goto = function(path, event, page){
        if (event) event.preventDefault();
        if (page) $scope.onPage = page;
        $location.path(path);
        return false;
      };

      $scope.logout = function() {
        navigator.vibrate(config.popups.confirmVibrateTime);
        $q.all({okText: $translate('POPUP_OK'), cancelText: $translate('POPUP_CANCEL'), title: $translate('POPUP_LOGOUT_TITLE'), text: $translate('POPUP_LOGOUT_TEXT')})
          .then(function(translations){
            var confirmPopup = $ionicPopup.confirm({
              title: translations.title,
              template: translations.text,
              okText: translations.okText,
              cancelText: translations.cancelText
            });
            confirmPopup.then(function(res) {
              if(res) {
                $scope.doLogout();
              }
            });
          })
          .catch(function(){
            var confirmPopup = $ionicPopup.confirm({
              title: 'POPUP_LOGOUT_TITLE',
              template: 'POPUP_LOGOUT_TEXT',
              okText: 'POPUP_OK',
              cancelText: 'POPUP_CANCEL'
            });
            confirmPopup.then(function(res) {
              if(res) {
                $scope.doLogout();
              }
            });
          });
      };
      $scope.shareApp = function() {
        if (window.plugins) {
          $translate('SHARE_TEXT').then(function (translation) {
            window.plugins.socialsharing.share(translation, null, null, 'http://discreetlyapp.com/');
          }).catch(function(){
            window.plugins.socialsharing.share('SHARE_TEXT', null, null, 'http://discreetlyapp.com/');
          });
        }
      };
      $scope.closeLogout = function() {
        $scope.modal.hide();
      };

      // Perform the login action when the user submits the login form
      $scope.doLogin = function(isSignup, e) {
        $scope.errors = {};

        var inputError = userProfile.hasInputErrors($scope.loginData, isSignup);

        if (inputError){
          $scope.errors = inputError;
          //console.log('doLogin errors', inputError);
          $scope.threeFails++;
          return;
        } else {
          $scope.loadButton(e);
        }

        userProfile
          .login($scope.loginData, isSignup)
          .then(function(data) {
            $scope.isLoggedIn = true;
            $rootScope.$broadcast('logged-in', data);
            if (data.profileComplete) {
              userProfile.loadInfo().then(function(data){
                $rootScope.$broadcast('profile-ready', data);
                $location.path('/app/matches');
              });
            } else {
              $location.path('/start/empty-profile');
            }
          })
          .catch(function(error) {
            $scope.unloadButton(e);
            $scope.error = 'FORM_ERROR_USER_ID_OR_PASSWORD_INVALID';
            //console.log('Error login', error);
          });
      };
      // Perform the login action when the user submits the login form
      $scope.doSignUp = function(isSignup, e) {
        $scope.errors = {};

        var inputError = userProfile.hasInputErrors($scope.loginData, isSignup);

        if (inputError){
          $scope.errors = inputError;
          //console.log('doLogin errors', inputError);
          $scope.threeFails++;
          return;
        } else {
          $scope.loadButton(e);
        }

        baseData
            .post(URL.SIGNUP, {
              countryCode: $scope.loginData.countryCode,
              phoneNumber: $scope.loginData.username
            }, null, 'no-ping-check')
            .then(function(){
              $location.path('/start/verify');
            })
            .catch(function(){
              $scope.unloadButton(e);
            });
      };
      $scope.doLogout = function() {
        $location.path('/app/force-logout');
      };
      $scope.feedback = function() {
        $q.all({
          cancelText: $translate('ACTION_MENU_CANCEL'),
          rateBtn: $translate('ACTION_MENU_FEEDBACK_RATE_BTN_TEXT'),
          emailBtn: $translate('ACTION_MENU_FEEDBACK_EMAIL_BTN_TEXT'),
          title: $translate('ACTION_MENU_FEEDBACK_TITLE'),
          emailText: $translate('ACTION_MENU_FEEDBACK_EMAIL_TEXT'),
          emailSubject: $translate('ACTION_MENU_FEEDBACK_EMAIL_SUBJECT'),
          emailAddy: $translate('ACTION_MENU_FEEDBACK_EMAIL')
        })
          .then(function(translate){
            var buttonObj = [];
            var emailIndex = -1;
            if (emailAvailable.get()) {
              buttonObj.push({ text: translate.emailBtn });
              emailIndex = 0;
            }
            buttonObj.push({ text: translate.rateBtn });
            var hideSheet = $ionicActionSheet.show({
              buttons: buttonObj,
              titleText: translate.title,
              cancelText: translate.cancelText,
              buttonClicked: function(index) {
                if (index == emailIndex) {
                  analytics.trackEvent('feedback', 'mail');
                  window.plugin.email.open({
                    to: [translate.emailAddy],
                    subject: translate.emailSubject,
                    body: '\n\n\n' + translate.emailText + '\n----------------------------\nID: ' + userProfile.getProfileId() + '\n----------------------------\n',
                    isHtml: false
                  });
                } else if (index == 1) {
                  // TODO: add app id's and package names
                  analytics.trackEvent('feedback', 'rate');
                  if (device.platform == 'iOS') {
                    window.open('itms-apps://itunes.apple.com/us/app/domainsicle-domain-name-search/id511364723?ls=1&mt=8'); // or itms://
                  } else if (device.platform == 'Android') {
                    window.open('market://details?id=<package_name>');
                  } else if (device.platform == 'BlackBerry'){
                    window.open('http://appworld.blackberry.com/webstore/content/<applicationid>');
                  }
                }
                return true;
              }
            });
          })
          .catch(function(){
            //console.log("** ERROR ** getting the translations for this action!");
          });
      };

      $scope.broadcast = function(eventName, data) {
        $rootScope.$broadcast(eventName, data);
      };

      $scope.addSelected = function(elId) {
        $scope.menuSelected = elId;
      };

      $scope.env = {
        isTest: config.env === 'test'
      };
      $scope.$watch(
        function() { return $scope.env.isTest; },
        function(newValue, oldValue) {
          $scope.setEnv(newValue);
        }
      );
      $scope.setEnv = function(isTest) {
        var env = isTest && 'test' || 'prod';
        //console.log('- ' + isTest + ' -> ' + env);
        config.setEnv(env);
      };

      // functions for making a button on load and off
      $scope.loadButton = function(e) {
        var target = e.target || e.srcElement;
        var elem = angular.element(target);
        var currentText = elem.text();
        elem.attr('data-oridinal-text', currentText);
        elem.html('<i class="icon ion-loading-d">');
        elem.attr('disabled', 'disabled');
      }
      $scope.unloadButton = function(e) {
        var target = e.target || e.srcElement;
        var elem = angular.element(target);
        elem.text(elem.attr('data-oridinal-text'));
        elem.removeAttr('disabled');
      }
    };

    return AppCtrl;
  });

}(define));