(function (define) {
  'use strict';

  define(['config'], function (config) {

    var EditProfileCtrl = function($scope, $rootScope, $stateParams, $q, userProfile, $location, uploadImageDialog, notificationPopover, contactListData, $ionicModal) {
      $scope.title = 'TITLE_START_PROFILE';
      $scope.profileComplete = false;
      $scope.loaded = false;
      $scope.profileId;
      $scope.imageId;
      $scope.profileImageControl = {};
      $scope.user = {};
      $scope.error = '';
      $scope.errors = {};
      $scope.messageDestructionDisplay = 0;
      $scope.user.uploading = false;

      // have to put it inside object because of the $watch
      $scope.m = {messageDestructionIndex: 0};
      // zero, 1 min, 10 min, 30 min, 1 hour, 6 hours, 24 hour, 3 days, 1 week, 2 weeks, one month
      var autoMessageDestruct = [0,60,600,1800,3600,21600,86400,259200,604800,1209600,2419200].map(function(i){return i*1000;});
      var currentTime = new Date().getTime();

      $scope.$watch(
        function(scope) {
          return scope.m.messageDestructionIndex;
        },
        function(newValue) {
          console.log('$watch: messageDestructionIndex = ' + newValue);
          $scope.messageDestructionDisplay = currentTime + autoMessageDestruct[newValue];
        }
      );

      notificationPopover.listenAndPopup($scope);

      $scope.setPage('singup');
      if (window.StatusBar) StatusBar.styleLightContent();

      userProfile.getLoginInfo()
        .then(function(loginInfo){
          $scope.profileId = loginInfo.profileId;
          $scope.profileComplete = loginInfo.profileComplete;
          if ($scope.profileComplete){
            $scope.title = 'TITLE_EDIT_PROFILE';
          }
          return userProfile.loadInfo();
        })
        .then(function(data){
          $scope.user = data;

          var index = autoMessageDestruct.indexOf($scope.user.messageDestruction);
          $scope.m.messageDestructionIndex = index === -1 ? 0 : index;

          $scope.user.imageId = data.profileImages && data.profileImages[0] && data.profileImages[0].imageId;
          $scope.loaded = true;
        })
        .catch(function(error){
          //console.log('Error: Cannot load user data: ', error);
          $scope.error = error;
        });

      $scope.logout = function(){
        userProfile.logout();
        $location.path('/start/first-run');
      };

      $scope.clearErrors = function(){
        $scope.error = '';
        $scope.errors = {};
      };

      $scope.saveProfile = function(data, e){
        $scope.loadButton(e);
        if (!$scope.user.uploading) {
          if (!data.genderPreferenceMale) data.genderPreferenceMale = false;
          if (!data.genderPreferenceFemale) data.genderPreferenceFemale = false;

          data.messageDestruction = autoMessageDestruct[$scope.m.messageDestructionIndex];

          fireSignificantChange(userProfile.getFull(), data);

          contactListData.getAndSave()
            .catch(function (error) {
              // TODO: remove setting private dating (incognito) if turned on and notify individual
              console.log(error);
            })
            .finally(function(){
              userProfile
                .save(data)
                .then(showContactsInfoAndProceed)
                .catch(function (errors) {
                  $scope.errors = errors;
                  $scope.unloadButton(e);
                });
            });

        } else {
          $scope.errors.imageId = 'PROFILE_ERROR_PHOTO_UPLOADING';
          $scope.unloadButton(e);
        }
      };
      function fireSignificantChange(oldSettings, newSettings) {
        if (oldSettings && (oldSettings.genderPreferenceMale !== newSettings.genderPreferenceMale || oldSettings.genderPreferenceFemale !== newSettings.genderPreferenceFemale)) {
          $rootScope.$broadcast(config.event.SIGNIFICANT_CHANGE);
        }
      }

      $scope.uploadPicture = function(){
        if ($scope.errors.imageId) $scope.clearErrors();
        uploadImageDialog.popup({ type: 'profile', onImageSuccess: function(data){
          $scope.user.imageId = null;
          $scope.user.preview = true;
          $scope.user.uploading = true;
          $scope.user.uploaded = false;
          $scope.user.profileImages = [{imageId: null}];
          $scope.profileImageControl.refresh && $scope.profileImageControl.refresh(null, data);
        }}).then(showPicture).catch(onPictureError).finally(function(){
          $scope.user.uploading = false;
          if ($scope.errors.imageId == 'PROFILE_ERROR_PHOTO_UPLOADING') $scope.saveProfile();
        });
      };

      function showContactsInfoAndProceed(data) {
        if (!$scope.profileComplete) {
          var deferred = $q.defer();
          var curDevice;
          if (window.device) {
            curDevice = device.platform;
          } else {
            curDevice = 'iOS';
          }
          if (curDevice == 'iOS') {
            $ionicModal.fromTemplateUrl('templates/dialogs/ios-why-contacts-modal.html', {
              scope: $scope,
              animation: 'slide-in-up'
            }).then(function (modal) {
              $scope.modal = modal;
              $scope.modal.show();
            });
            $scope.closeModal = function () {
              deferred.resolve();
              $scope.modal.hide();
            };
            $scope.$on('$destroy', function () {
              $scope.modal.remove();
            });
          } else {
            deferred.resolve();
          }

          deferred.promise.then(function () {
            $rootScope.$broadcast('profile-ready', data);
            $location.path('/app/matches');
          });
        } else {
          var userFullInfo = userProfile.getFull();
          $scope.changeFirstName(userFullInfo.firstName);

          $location.path('/app/my-profile');
        }
      }

      function showPicture(imageId){
        $scope.user.imageId = imageId;
        $scope.user.uploaded = true;
        delete $scope.errors.imageId;
        $scope.user.profileImages = [{imageId: imageId}];
        $scope.profileImageControl.refresh && $scope.profileImageControl.refresh(imageId);
        userProfile.loadInfo(true).then(function(){
          $rootScope.$broadcast('profile-image-updated', {imageId: imageId});
        });
      }
      function onPictureError(error){
        $scope.errors.imageId = 'PROFILE_ERROR_PHOTO_UPLOAD';
      }
    };

    return EditProfileCtrl;
  });

}(define));