(function (define) {
  'use strict';

  define(['config'], function (config) {

    function openTerms() {
      window.open(config.termsUrl, '_system');
    }

    var SettingsCtrl = function($scope, $q, settingsData, userProfile, $ionicPopup, $ionicPopover, notificationPopover, $location, $translate, amMoment){
      $scope.version = config.version;
      $scope.env = config.env === 'prod' ? 'Production' : 'Test';
      $scope.loaded = false;
      $scope.languages = config.lang.languagesTitles;
      $scope.openTerms = openTerms;
      // have to put it inside object because of the $watch
      $scope.m = {messageDestructionIndex: 0};
      // zero, 1 min, 10 min, 30 min, 1 hour, 6 hours, 24 hour, 3 days, 1 week, 2 weeks, one month
      var autoMessageDestruct = [0,60,600,1800,3600,21600,86400,259200,604800,1209600,2419200].map(function(i){return i*1000;});
      var currentTime = new Date().getTime();

      var currentLanguage = $translate.use();
      if (config.lang.localeCodes.indexOf(currentLanguage) != -1) {
        $scope.selectedLang = config.lang.languagesTitles[config.lang.localeCodes.indexOf(currentLanguage)];
      } else {
        //console.log('error retrieving current language');
      }

      notificationPopover.listenAndPopup($scope);

      $ionicPopover.fromTemplateUrl('templates/dialogs/language-popover.html', {
        scope: $scope
      }).then(function(popover) {
        $scope.popover = popover;
        $scope.selectLanguage = function(index) {
          //console.log("Selected Language " + $scope.languages[index]);
          $scope.selectedLang = $scope.languages[index];
          $translate.use(config.lang.localeCodes[config.lang.languagesTitles.indexOf($scope.languages[index])]);
          amMoment.changeLocale(config.lang.localeCodes[config.lang.languagesTitles.indexOf($scope.languages[index])]);

          $scope.popover.hide();
        }
      });

      $scope.popoverLanguage = function($event){
        $scope.popover.show($event);
      };
      $scope.$on('$destroy', function() {
        $scope.popover.remove();
      });

      settingsData.get().then(function(data){
        $scope.settings = data;
        var index = autoMessageDestruct.indexOf($scope.settings.messageDestruction);
        $scope.m.messageDestructionIndex = index === -1 ? 0 : index;
        $scope.loaded = true;
      });
      $scope.$watch(
        function(scope) {
          return scope.m.messageDestructionIndex;
        },
        function(newValue) {
          $scope.messageDestructionDisplay = currentTime + autoMessageDestruct[newValue];
        }
      );
      $scope.saveSettings = function(){
        delete $scope.settings['updatedTimestamp'];
        $scope.settings['messageDestruction'] = autoMessageDestruct[$scope.m.messageDestructionIndex];
        settingsData.save($scope.settings).then(function(){
          $ionicPopup.alert({
            title: 'Settings',
            template: 'Your Settings Have Been Saved'
          });
        });
      };
      $scope.destroyAccount = function(){
        navigator.vibrate(config.popups.confirmVibrateTime);
        $q.all({okText: $translate('POPUP_OK'), cancelText: $translate('POPUP_CANCEL'), title: $translate('POPUP_SETTINGS_DESTROY_TITLE'), text: $translate('POPUP_SETTINGS_DESTROY_TEXT')})
          .then(function(translations){
            var confirmPopup = $ionicPopup.confirm({
              title: translations.title,
              template: translations.text,
              okText: translations.okText,
              cancelText: translations.cancelText
            });
            confirmPopup.then(function(res) {
              if(res) {
                //console.log('Destroying account 1 ...');
                settingsData.destroy().then(function(){
                  $location.path('/app/force-logout');
                });
              }
            });
          })
          .catch(function(){
            var confirmPopup = $ionicPopup.confirm({
              title: 'POPUP_SETTINGS_DESTROY_TITLE',
              template: 'POPUP_SETTINGS_DESTROY_TEXT',
              okText: 'POPUP_OK',
              cancelText: 'POPUP_CANCEL'
            });
            confirmPopup.then(function(res) {
              if(res) {
                //console.log('Destroying account 2 ...');
                settingsData.destroy().then(function(){
                  $location.path('/app/force-logout');
                });
              }
            });
          });
      };
    };

    return SettingsCtrl;
  });

}(define));