/**

[{
  "chatId": "54302a23d944b0a03415957c",
  "digitalAssets": [],
  "lastMessage": "Welcome to Kismet!",
  "lastMessageId": "9c0998a3-ac18-4274-892d-d4dba1eb9646",
  "lastMessageFromYou": true,
  "lastMessageTimeStamp": 1414172278910,
  "hasNewMessage": false,
  "isNewMatch": false,
  "participant": {
    "profileId": "62806595-1340-4c9e-ba5c-c2d8a1eb6a56",
    "firstName": "IOS",
    "imageId": "b0144570-4be5-11e4-8701-f7437c5af76d",
    "lastActive": 1412447478052
  }
}, ...]

 **/

(function (define, angular) {
  'use strict';

  define(['config','utils'], function (config, utils) {

    var _ = utils._;

    var chatsCollection = function (collectionData, $rootScope, $q, baseData) {
      var apiUrl = '/api/chat/active-chats?profileId={profileId}',
        baseInstance = collectionData({
          name: 'chatlist',
          url: apiUrl,
          idProp: 'chatId',
          sortBy: 'lastMessageTimeStamp'
        });

      var instance = angular.extend(baseInstance, {
        updateChat: function(options) {
          // chatId, lastMessage, lastMessageFromYou, lastMessageTimeStamp, hasNewMessage
          var chat = instance.get(options.chatId),
            chatOptions = angular.extend({
              isNewMatch: false,
              lastMessageTimeStamp: (new Date).getTime()
            }, options);
          if (chat) {
            chat = angular.extend(chat, chatOptions);
          }
          baseInstance.fireUpdate();
          return chat;
        },
        countUnreadChats: function() {
          var count = baseInstance._getAll().reduce(function(acc, cur){ return cur.hasNewMessage || cur.isNewMatch ? ++acc : acc }, 0);
          return count;
        },
        blockUser: blockUser
      });

      window.Kismet.collections.chatsCollection = instance;

      $rootScope.$on('chat-new-message', function(e, data){
        console.log('on chat-new-message', data);
        instance.updateChat({
          chatId: data.chatId,
          lastMessage: data.message && data.message.text,
          lastMessageFromYou: data.message && data.message.isMine,
          lastMessageTimeStamp: data.message && data.message.timestamp,
          hasNewMessage: data.hasNewMessage !== 'undefined' ? data.hasNewMessage : true,
          isNewMatch: data.isNewMatch !== 'undefined' ? data.isNewMatch : false
        });
      });
      $rootScope.$on(config.event.CHAT_WIPE, onChatWipe);
      $rootScope.$on(config.event.CHAT_MESSAGE_DELETED, onChatMessageDeleted);

      function blockUser(chatId, myProfileId) {
        var deferred = $q.defer(),
          chat = baseInstance.get(chatId),
          participantId = chat && chat.participant && chat.participant.profileId;

        baseData.post('/api/profile/blacklist?profileId={profileId}', {blacklistProfileId: participantId}, myProfileId)
          .then(function(){
            instance.remove(chat, true);
            deferred.resolve('OK');
          }).catch(function(err){
            deferred.reject('Sorry, server error. ' + err);
          });

        return deferred.promise;
      }
      function onChatWipe(e, data){
        var chat = baseInstance.get(data.chatId);
        if (!chat.lastMessageFromYou) {
          chat.lastMessage = '';
          baseInstance.fireUpdate();
        }
      }
      function onChatMessageDeleted(e, data){
        var chat = baseInstance.get(data.chatId);
        if (!chat.lastMessageFromYou && chat.lastMessageId === data.message.messageId) {
          chat.lastMessage = '';
          baseInstance.fireUpdate();
        }
      }

      return instance;
    };

    return chatsCollection;
  });
}(define, angular));