(function (define) {
  'use strict';

  define(['config', 'utils'], function (config, utils) {

    var _ = utils._;
    if (!window.Kismet) window.Kismet = {};
    window.Kismet.collections = {};

    var collectionData = function ($q, $rootScope, listData, baseData) {
      window.Kismet.baseData = baseData;

      return function(options) {

        var items = [],
          name = options.name,
          apiUrl = options.url,
          idProp = options.idProp || 'id',
          defaultSort = options.sortBy,
          updateEventName = 'collection-' + name + '-update',
          wasLoaded = false,
          shouldLoadOnTake = true;

        $rootScope.$on('logout', function(){
          wipe();
        });

        function take(mnt, start) {
          var resultItems = items;
          //console.log('collection.take: shouldLoadOnTake=' + shouldLoadOnTake);
          if (shouldLoadOnTake) {
            load();
          }
          if (defaultSort) {
            resultItems = _.sortBy(_.prop(defaultSort))(items);
          }
          if (mnt) {
            resultItems = _.take(mnt, resultItems);
          }
          return resultItems;
        }
        function get(id) {
          var spec = {}; spec[idProp] = id;
          var item = _.find(_.where(spec), items);
          // TODO: add to error-reporting
          if (!item && id != 'typing') {
            console.log('WARNING: cant find item by id [' + id + ']');
          }
          return item;
        }
        function last() {
          return items.length && items[items.length - 1];
        }
        function isLast(message) {
          return items.indexOf(message) === items.length - 1;
        }
        function add(item, fn, front) {
          var spec = {}; spec[idProp] = item[idProp];
          if (!_.find(_.where(spec), items) && (typeof fn !== 'function' || fn(item, items))) {
            items[front && 'unshift' || 'push'](item);
          } else {
            console.log('!!! WARNING: Collection tried to add a duplicate', item);
          }
          return items;
        }
        function unshift(items) {
          items.reverse().forEach(function(i){
            add(i, null, true);
          });
        }
        function remove(itemOrId, shouldFireUpdate) {
          var item = typeof itemOrId === 'object' ? itemOrId : get(itemOrId);
          item && items.splice(items.indexOf(item), 1);
          //$rootScope.$broadcast(updateEventName, items);
          item && shouldFireUpdate && fireUpdate();
          return item;
        }
        function onUpdate(scope, callback) {
          callback && scope && scope.$on && scope.$on(updateEventName, function(event, data) {
            //console.log('collection.onUpdate: captured EVENT ' + updateEventName);
            callback(data);
          });
        }
        function fireUpdate(data) {
          //console.log('collection.load: fire ' + updateEventName);
          $rootScope.$broadcast(updateEventName, data || items);
        }
        function load(opt) {
          //console.log('collection.load: ' + name, opt);
          shouldLoadOnTake = false;
          var deferred = $q.defer(),
            url = opt && opt.url || apiUrl;

          listData(url, 'Collection-' + name, {ttl: 0})
            .get()
            .then(function(data) {
              wasLoaded = true;
              if (options && options.preprocessData) {
                data = options.preprocessData(data);
              }
              //console.log('collection.load: loaded items ' + data.length);
              if (opt && opt.dataIncrement && !opt.noMutation) {
                //console.log('- option: dataIncrement');
                data.forEach(add);
              } else if (opt && opt.addUpTo && !opt.noMutation) {
                //console.log('- opt: addUpTo');
                if (items.length < opt.addUpTo) {
                  //console.log('- adding');
                  _.take(opt.addUpTo - items.length, data).forEach(add);
                }
              } else if (!(opt && opt.noMutation)) {
                //console.log('- opt: none');
                items = data;
              }
              if (!opt || !opt.skipEvent){
                fireUpdate(data);
              }
              deferred.resolve(data);
            })
            .catch(function(err){
              deferred.reject(err);
            });

          return deferred.promise;
        }
        function sync() {
          // ???
        }
        function markForUpdateOnTake() {
          //console.log('collection.markForUpdateOnTake: ' + name);
          shouldLoadOnTake = true;
          baseData.clearCache(apiUrl);
        }
        function wipe(){
          items = [];
          wasLoaded = false;
          shouldLoadOnTake = true;
          fireUpdate([]);
        }

        return {
          take: take,
          get: get,
          last: last,
          isLast: isLast,
          add: add,
          unshift: unshift,
          remove: remove,
          onUpdate: onUpdate,
          fireUpdate: fireUpdate,
          load: load,
          sync: sync,
          clear: markForUpdateOnTake,
          wipe: wipe,
          wasLoaded: function(){ return wasLoaded; },
          shouldLoadOnTake: function(){ return shouldLoadOnTake; },
          setItems: function(_items){
            items = _items;
            wasLoaded = true;
          },
          _getAll: function() {
            return items;
          }
        };

      }
    };

    return collectionData;
  });
}(define));


/*
    Sample usage:

    function Ctrl($scope, collection) {
      $scope.items = collection.take(10);
      collection.load(function(items){
        $scope.items = items;
      });
      collection.onUpdate($scope, function(items){
        $scope.items = items;
      });
    }


 */