(function (define) {
  'use strict';

  define(['config'], function (config) {

    var contactList = function ($q, baseData, userProfile) {

      var getContacts = function() {
        console.log('[getContacts]');

        if (typeof window['ContactFindOptions'] === 'undefined') {
          return $q.reject('ContactFindOptions does not exist in global.');
        }
        var options, deferred, fetchFields = [navigator.contacts.fieldType.name];
        try {
          options = new ContactFindOptions();
          deferred = $q.defer();
          fetchFields = [navigator.contacts.fieldType.name];
        } catch(e){
          console.log('ERROR: cannot retrieve contacts: ' + JSON.stringify(e));
          return $q.reject(e);
        }

        options.filter = "";
        options.multiple = true;
        options.desiredFields = [navigator.contacts.fieldType.phoneNumbers, navigator.contacts.fieldType.name];

        function deleteNulls(test, recurse){
          for (var i in test){
            if (test[i] === null){
              delete test[i];
            } else if (recurse && typeof test[i] === 'object'){
              deleteNulls(test[i], recurse);
            }
          }
        }

        function onSuccess(returnedContacts) {
          deleteNulls(returnedContacts, true);
          returnedContacts = returnedContacts.filter(function(contact){
            return typeof contact.phoneNumbers !== 'undefined';
          });
          console.log('- contacts found ' + returnedContacts.length + ', 1st:', JSON.stringify(returnedContacts[0]));
          deferred.resolve(returnedContacts);
        }

        function onError(contactError) {
          // TODO: handle this error
          console.log("Contact List Error");
          deferred.reject(contactError);
        }

        console.log('- navigator.contacts.find()...');
        navigator.contacts.find(fetchFields, onSuccess, onError, options);

        return deferred.promise;
      };

      var getAndSaveContacts = function() {
        function numbersOnly(string) {
          return string.replace(/[^0-9]/g, '');
        }
        return getContacts()
            .then(function(data){
              var newArr = [];
              for (var x = 0 ; x < data.length; x++) {
                if (typeof data[x].phoneNumbers !== 'undefined') {
                  for (var y = 0 ; y < data[x].phoneNumbers.length; y++) {
                    newArr.push(numbersOnly(data[x].phoneNumbers[y].value));
                  }
                }
              }
              console.log("Contacts ", newArr);
              if (newArr.length > 0){
                return baseData.post('/api/profile/contact-list?profileId={profileId}', {contacts:newArr}, userProfile.getProfileId());
              }else{
                return $q.reject("No contacts");
              }
            });
      };

      return {
        get: getContacts,
        getAndSave: getAndSaveContacts
      }
    };

    return contactList;
  });
}(define));