
(function (define) {
  'use strict';

  define(['config'], function (config) {

    var imageData = function($q, baseData, userProfile, security) {

      var imgCache = {},
        apiUrl = config.apiUrls.IMAGE,
        apiUrlBase64 = apiUrl.replace('{type}', 'base64'),
        apiUrlBinary = apiUrl.replace('{type}', 'binary');

      var getImage = function(imageId, size) {
        var cacheId = 'image-' + imageId + '-' + size;
        if (!imgCache[cacheId])
          imgCache[cacheId] = baseData.get(
            apiUrlBase64
              .replace('{profileId}', userProfile.getProfileId())
              .replace('{imageId}', imageId)
              .replace('{size}', size),
            {
              ttl: config.ttl.IMAGES
            }
          );
        return imgCache[cacheId];
      };

      var getThumb = function(imageId) {
        return getImage(imageId, 'thumb');
      };

      var getImageFromCacheMethod = function (imageId, size)
      {
        var cacheId = 'image-' + imageId + '-' + size;
        return imgCache[cacheId];
      };

      var getImageUrl = function(imageId, size) {
        var url = config.server.url + apiUrlBinary
          .replace('{imageId}', imageId)
          .replace('{size}', size);

        return url + security.bewit(url, config.ttl.FULL_IMAGE * 1000);
      };

      return {
        getThumb: getThumb,
        getImage: getImage,
        getImageUrl: getImageUrl,
        getImageFromCacheMethod:getImageFromCacheMethod
      };
    };

    return imageData;
  })
}(define));

