/**
 * Paged list
 */

(function (define) {
  'use strict';

  define(['config'], function (config) {

    var listData = function($q, baseData, userProfile) {

      return function (apiUrl, localStorageItemName, options) {

        var pagesDeferred = [],
          pageSize = options && options.pageSize || config.pageSize || 30,
          _pageNumber = 0,
          hasMore = $q.defer(),
          TTL = options && options.ttl;


        var getPage = function(pageNumber, opt) {
          opt = opt || {};
          opt.profileId = userProfile.getProfileId();
          if (typeof TTL !== 'undefined') {
            opt.ttl = TTL;
          }

          if (options && options.newDataEvent){
            opt.newDataEvent = options.newDataEvent;
          }

          return baseData.get(apiUrl, opt);
        };

        var next = function(pageNumber, opt) {
          pageNumber = typeof pageNumber !== 'undefined' ? pageNumber : _pageNumber++;
          return getPage(pageNumber, opt);
        };

        var clear = function(){
          baseData.clearCache(apiUrl);
        };

        var reset = function(){
          baseData.resetCache(apiUrl);
        };

        return {
          get: function (opt) {
            return next(0, opt);
          },
          next: next,
          clear: clear,
          reset: reset
        };
      }
    };


    return listData;
  })
}(define));