/**

[{
  "distance": 0.03244672205737491,
  "profileId": "5c700032-3816-4bea-bbdc-372f986c64a4",
  "firstName": "Mariam",
  "age": 18,
  "lastActive": 1412624057979,
  "img": "02b62850-4d8c-11e4-acd4-d9a7721e081f",
  "blurbs": [
    {
      "title": "",
      "text": "bla bla :)"
    }
  ]
}, ...]

**/

(function (define) {
  'use strict';

  define(['config', 'utils'], function (config, utils) {

    var _ = utils._;

    var matchesData = function ($q, $rootScope, $timeout, listData, baseData, userProfile) {

      var matches = [],
        cacheForPageCtrl,
        apiUrl = config.apiUrls.MATCH_RECOMMEND,
        likeUrl = config.apiUrls.MATCH_LIKE,
        undoUrl = config.apiUrls.MATCH_UNDO,
        refreshUrl = config.apiUrls.MATCH_REFRESH,
        loadPageSize = config.data.matchesLoadPageSize,
        loadPageNumber = -1,
        firstBatch = true,
        firstAfterLogin = true,
        firstAfterLoginCount = 0,
        firstAfterLoginDeferred,
        firstAfterLoginTimeout,
        undoObj = {
          user: null,
            control: {},
          processing: false
        };

      $rootScope.$on('logout', function(){
        matches = [];
        loadPageNumber = -1;
        cacheForPageCtrl = null,
        firstBatch = true;
        firstAfterLogin = true;
        firstAfterLoginCount = 0;
        undoObj.user = null;
      });
      $rootScope.$on(config.event.NEW_MATCH, function(event, data){
        cleanupOnNewMatch(data.participants && data.participants[0]);
      });
      $rootScope.$on(config.event.SIGNIFICANT_CHANGE, function(event, data){
        console.log('MatchesData: received SIGNIFICANT_CHANGE event');
        firstAfterLogin = true;
        firstAfterLoginCount = 0;
        cacheForPageCtrl = null;
      });

      var get = function(startIndex, amount, force) {
        var deferred = $q.defer();

        if (matches.length >= startIndex + amount && !force) {
          deferred.resolve(matches.slice(startIndex, startIndex + amount));
        } else {
          loadPageNumber++; // TODO: remove this obsolete.
          listData(apiUrl.replace('{firstBatch}', (firstBatch ? 'true' : 'false')).replace('{batchSize}', loadPageSize), 'Matches-' + loadPageNumber, {ttl: 0})
            .get()
            .then(function(data) {
              if (startIndex === 0) {
                matches = [];
              }
              if (data && data.forEach) {
                data.forEach(function(item){
                  if (item.profileId && !_.find(_.where({profileId: item.profileId}))(matches)) {
                    matches.push(item);
                  }
                });
              }
              deferred.resolve(matches.slice(startIndex, startIndex + amount));
            })
            .catch(function(error){
              // errorCode: "E004", errorMessage: "No geo location set for profile."
              deferred.reject(error);
            });
          firstBatch = false;
        }
        return deferred.promise;
      };

      var remove = function(item) {
        item && matches.splice(matches.indexOf(item), 1);
        item && cacheForPageCtrl && cacheForPageCtrl.splice(cacheForPageCtrl.indexOf(item), 1);
      };
      var removeAll = function(){
        matches = [];
      };

      var sendLike = function(match, liked) {
        var url = likeUrl
          .replace('{profileId}', userProfile.getProfileId())
          .replace('{matchedId}', match.profileId)
          .replace('{liked}', liked);

        return baseData.post(url);
      };

      var like = function(match) {
        return sendLike(match, 'true');
      };

      var dislike = function(match) {
        return sendLike(match, 'false');
      };

      var refresh = function() {
        var url = refreshUrl.replace('{profileId}', userProfile.getProfileId());
        return baseData.post(url);
      };

      function getAfterLogin(amount) {
        //console.log('getAfterLogin: try# ' + firstAfterLoginCount);
        firstBatch = true;
        get(0, amount, true).then(function(matches){
          if (matches.length === 0 && firstAfterLoginCount < 5) {
            console.log('getAfterLogin: ZERO after try# ' + firstAfterLoginCount + ': ' + matches.length);
            firstAfterLoginTimeout = $timeout(function(){
              firstAfterLoginCount++;
              getAfterLogin(amount);
            }, 15*1000);
          } else {
            firstAfterLogin = false;
            console.log('getAfterLogin: SUCCESS after try# ' + firstAfterLoginCount + ': ' + matches.length);
            firstAfterLoginDeferred.resolve(matches);
          }
        }).catch(function(error){
          if (firstAfterLoginTimeout) {
            $timeout.cancel(firstAfterLoginTimeout);
          }
          console.log('ERROR: getAfterLogin: rejected after try# ' + firstAfterLoginCount + '. Error msg:' + JSON.stringify(error));
          firstAfterLoginDeferred.reject(error);
        });
      }
      function undo(item, index) {
        if (matches.indexOf(item) === -1) {
          var url = undoUrl
            .replace('{profileId}', userProfile.getProfileId())
            .replace('{matchedId}', item.profileId);

          return baseData
            .post(url).then(function(){ return item; })
            .then(function(result){
              matches.splice((index || 0), 0, item);
              return result;
            });
        }
        return $q.reject();
      }
      function getMatchById(profileId){
        return _.find(_.where({profileId: profileId}))(matches);
      }
      function cleanupOnNewMatch(participant){
        if (participant && undoObj.user && undoObj.user.profileId === participant.profileId) {
          undoObj.user = null;
        }
        participant && remove(getMatchById(participant.profileId));
      }

      return {
        get: function(amount, options) {
          if (options && options.tryCache && (cacheForPageCtrl || matches.length >= amount)) {
            return cacheForPageCtrl || matches.slice(0, amount);
          }
          if (firstAfterLogin || options && options.refresh) {
            var user = userProfile.getFull();
            //console.log('matchesData: get: firstTimeRegistering=' + user.firstTimeRegistering, user, amount, options);
            if (firstAfterLoginTimeout) {
              $timeout.cancel(firstAfterLoginTimeout);
            }
            firstAfterLoginDeferred = $q.defer();
            getAfterLogin(amount);
            return firstAfterLoginDeferred.promise;
          }
          return get(0, amount);
        },
        next: function(startIndex, amount) {
          return get(startIndex, amount);
        },
        remove: remove,
        removeAll: removeAll,
        like: like,
        dislike: dislike,
        getMatchById: getMatchById,
        _getAll: function(){
          return matches.map(function(a){return a});
        },
        refresh: refresh,
        undo: undo,
        undoObj: undoObj,
        cleanupOnNewMatch: cleanupOnNewMatch,
        saveCache: function(matches){
          cacheForPageCtrl = matches;
        }
      };
    };

    return matchesData;
  });
}(define));