/**

 {
   "chatId": "54302a23d944b0a03415957c",
   "participants": [{
     "profileId": "62806595-1340-4c9e-ba5c-c2d8a1eb6a56",
     "firstName": "IOS",
     "imageId": "b0144570-4be5-11e4-8701-f7437c5af76d"
   },{
     "profileId": "a54aef35-796a-4b9c-9ea9-f7928207094a",
     "firstName": "Lemonchella",
     "imageId": "73efa010-4be7-11e4-83ac-e3bf77a58a4a"
   }],
   "messages": [{
     "messageId": "c720417f-8487-4ea3-b57b-59b6d2a430e8",
     "from": "62806595-1340-4c9e-ba5c-c2d8a1eb6a56",
     "text": "Hello Lemonchella ... ",
     "timestamp": 1412442659073,
     "destructionTimestamp": "",
     "state": "read",
     "location": {lat:75.5656, long: 34.45645},
     "digitalAssets": [{
       "id": "dsfg"
       "type": "Image| Video | Voice"
     }]
   }, ...]
 }

 **/


(function (define, angular) {
  'use strict';

  define(['config','utils'], function (config, utils) {

    var _ = utils._;

    var messageCollection = function ($rootScope, $timeout, $q, collectionData, $window, listData, userProfile, $sceDelegate, $sce, security, transportBroker) {
      var apiUrl = '/api/chat/messages?chatId={chatId}&profileId={profileId}',
        instances = {};

      $window.Kismet.collections.messageCollection = instances;

      return function(chatId, myProfileId, options) {
        if (options && options.isExisting && !instances[chatId]) {
          return;
        }

        if (instances[chatId]) {
          //console.log('messageCollection: existing collection for ' + chatId + ' with ' + instances[chatId]._getAll().length);
          return instances[chatId];
        }
        console.log('messageCollection: creating a new collection for ' + chatId);

        var chatUrl = apiUrl.replace('{chatId}', chatId),
          hasMoreMessages,
          isNewMatch,
          createdTimestamp,
          participants,
          baseInstance = collectionData({
            name: 'messages-' + chatId,
            url: chatUrl,
            idProp: 'messageId',
            sortBy: 'lastMessageTimeStamp',
            preprocessData: preprocessData
          });

        var instance = angular.extend({}, baseInstance, {
          items: function(){
            var items = baseInstance._getAll();
            console.log('MessageCollection for ' + chatId + ': contains ' + items.length);
            return items;
          },
          load: load(chatUrl),
          clear: function(){
            listData(chatUrl).clear();
          },
          hasMoreMessages: function() { return hasMoreMessages; },
          isNewMatch: function() { return isNewMatch; },
          createdTimestamp: function() { return createdTimestamp; },
          getParticipants: function(){ return participants; },
          getChatId: function(){ return chatId; },

          take: function(number, force){
            var items = _.compose(_.reverse, _.take(number), _.reverse, _.sortBy(_.prop('lastMessageTimeStamp')))(baseInstance._getAll());
            //console.log('messageCollection.take(' + number + '): ' + items.length);
            if (items.length === 0 || force) {
              //console.log('- load as first...');
              baseInstance.load();
              return items;
            }
            if (items.length < number && hasMoreMessages) {
              var messageId = items[0].messageId;
              //console.log('- load more starting with ' + messageId + ' (hasMoreMessages=' + hasMoreMessages + ')');
              baseInstance.load({
                url: chatUrl + '&lastMessageId=' + messageId,
                addUpTo: number
              });
            }
            return items;
          },
          sync: sync,
          tryRemove: tryRemove,
          tryAdd: tryAdd,
          tryAddRequest: tryAddRequest,
          messageAsRead: messageAsRead,
          onChange: onChange
        });

        instances[chatId] = instance;

        $rootScope.$on(config.event.CHAT_MESSAGE_NEW + '-' + chatId, onMessageNew);
        $rootScope.$on(config.event.CHAT_MESSAGE_SENT + '-' + chatId, onMessageSent);
        $rootScope.$on(config.event.CHAT_MESSAGE_DELIVERED + '-' + chatId, onMessageDelivered);
        $rootScope.$on(config.event.CHAT_MESSAGE_READ + '-' + chatId, onMessageRead);
        $rootScope.$on(config.event.CHAT_MESSAGE_DELETED + '-' + chatId, onMessageDeleted);
        $rootScope.$on(config.event.CHAT_MESSAGE_TYPING + '-' + chatId, onMessageTyping);
        $rootScope.$on(config.event.CHAT_WIPE + '-' + chatId, onChatWipe);

        function preprocessData(data) {
          hasMoreMessages = data.hasMoreMessages;
          participants = data.participants;
          createdTimestamp = data.createdTimestamp;
          isNewMatch = data.isNewMatch;
          return data.messages.filter(function(m){return m.state !== 'deleted';});
        }

        function sync(lastMessageId, amount) {
          return instance.load(lastMessageId, amount).then(normalize).then(lastMessageId && updateMore || update);
        }
        function normalize(messages) {
          //console.log('normalize:', messages);
          var arr = messages.map(normalizeItem(myProfileId, participants, $sceDelegate, $sce, security, $q));
          return arr;
        }
        function update(items) {
          console.log('collection.update: current items: ' + _dumpText(instance.items()));
          if (items && items.length > 0) {
            instance.remove('typing');
            var _items = _.clone(instance.items());
            _items.length > 0 && (console.log('collection.update: _items: ' + _items.length + ' ' + _dumpText(_items)), _items.forEach(instance.remove));
            items.forEach(function(m){instance.add(m, checkAssets)});
            _items.length > 0 && _items.forEach(instance.add);
            console.log('collection.update: items: ' + _dumpText(instance.items()));
          }
          instance.remove('typing');
          return instance.items();
        }
        function updateMore(items) {
          instance.unshift(items);
          return instance.items();
        }
        function _dumpText(msgs) {
          return _.map(_.prop('text'), msgs).join(', ');
        }
        function load(chatUrl) {
          return function(messageId, amount){
            var opt = messageId && {
                  url: chatUrl + '&lastMessageId=' + messageId,
                  addUpTo: 10,
                  skipEvent: true,
                  noMutation: true
                } || {noMutation: true};     //, url: chatUrl + (amount > 10 && '&pageSize=' + amount || '')  // there is no such an option on serverside.
            console.log('load:', opt);
            return baseInstance.load(opt);
          }
        }
        var typingMessageItem, timeoutPromise;
        function onMessageNew(event, data) {
          data.isMine = false;
          data.state = 'queued';
          var messageItem = normalizeItem(myProfileId, participants, $sceDelegate, $sce, security, $q)(data);
          //console.log('[messageCollection] onMessageNew', data);
          typingMessageItem = baseInstance.get('typing') || null;
          timeoutPromise && $timeout.cancel(timeoutPromise);
          if (!typingMessageItem){
            baseInstance.add(messageItem);
            //console.log('just new item (no typing)');
          } else {
            if (baseInstance.get(messageItem.messageId)) {
              console.log('WARNING: dup got captured along with typing.');
            } else {
              Object.keys(messageItem).forEach(function(key){
                typingMessageItem[key] = messageItem[key];
              });
            }
            typingMessageItem = null;
          }
          transportBroker.delivered(messageItem.messageId, chatId);
          $rootScope.$broadcast('msg-collection-change-' + chatId, {type: 'new-item'});
          $rootScope.$broadcast('chat-new-message', {chatId: chatId, message: messageItem});

        }
        function onMessageSent(event, data) {
          var message = baseInstance.get(data.messageId);
          if (message) {
            message.state = 'queued';
            message.displayState = getDisplayState('queued');
            data.timestamp && (message.timestamp = data.timestamp);
            $rootScope.$broadcast('msg-collection-change-' + chatId, {type: 'item-state'});
          }
        }
        function onMessageDelivered(event, data) {
          var message = baseInstance.get(data.messageId);
          if (message && (message.state === 'sent' || message.state === 'queued')){
            message.state = 'delivered';
            message.displayState = getDisplayState('delivered');
            data.timestamp && (message.timestamp = data.timestamp);
            $rootScope.$broadcast('msg-collection-change-' + chatId, {type: 'item-state'});
          }
        }
        function onMessageRead(event, data) {
          var message = baseInstance.get(data.messageId);
          if (message && message.state !== 'read' && message.state !== 'deleted'){
            message.state = 'read';
            message.displayState = getDisplayState('read');
            data.timestamp && (message.timestamp = data.timestamp);
            $rootScope.$broadcast('msg-collection-change-' + chatId, {type: 'item-state'});
          }
        }
        function onChatWipe(event, data) {
          var messages = baseInstance._getAll();
          messages.filter(function(m){return !m.isMine;}).forEach(_removeMessageAnimated);
        }
        function onMessageDeleted(event, data) {
          var message = baseInstance.get(data.messageId);
          if (message) {
            _removeMessageAnimated(message);
          }
        }
        function _removeMessageAnimated(message) {
          message.state = 'deleted';
          message.displayState = getDisplayState('deleted');
          $rootScope.$broadcast('msg-collection-change-' + chatId, {type: 'item-state'});
          $timeout(function(){
            var isLast = baseInstance.isLast(message);
            baseInstance.remove(message);
            if (isLast) {
              $rootScope.$broadcast('chat-new-message', {chatId: chatId, message: {chatId: chatId, hasNewMessage: false, message: baseInstance.last()}});
            }
          }, 200);
        }
        function onMessageTyping(event, data) {
          var diff = moment().diff(data.timestamp, 'seconds');
          if (diff && diff > 300) {
            console.log('outdated typing message (more than 300sec). ' + diff);
            return;
          }

          typingMessageItem = baseInstance.get('typing');
          //console.log('messageCollection.onMessageTyping: ' + JSON.stringify(typingMessageItem));
          if (!typingMessageItem) {
            data.messageId = 'typing';
            data.state = 'typing';
            data.isMine = false;
            data.text = 'typing';
            typingMessageItem = normalizeItem(myProfileId, participants, $sceDelegate, $sce, security, $q)(data);
            baseInstance.add(typingMessageItem);
            //console.log('messageCollection.onMessageTyping: added typing item', baseInstance._getAll());
            $rootScope.$broadcast('msg-collection-change-' + chatId, {type: 'new-item'});
          }
          timeoutPromise && $timeout.cancel(timeoutPromise);
          timeoutPromise = $timeout(function(){
            //console.log('removing typing on timeout');
            baseInstance.remove('typing');
            typingMessageItem = null;
          }, config.ttl.CHAT_TYPING_BUBBLE_SEC * 1000);
        }
        function tryAdd(data, noRequest) {
          //console.log('[messageCollection.tryAdd]', data);
          if (!data.messageId) {
            data.messageId = uuid4.generate();
          }
          var messageItem = normalizeItem(myProfileId, participants, $sceDelegate, $sce, security, $q)(data);
          if (typingMessageItem) {
            baseInstance.remove('typing');
          }
          baseInstance.add(messageItem);
          if (typingMessageItem) {
            baseInstance.add(typingMessageItem);
          }
          $rootScope.$broadcast('msg-collection-change-' + chatId, {type: 'new-item-mine', isTyping: !!typingMessageItem});
          !noRequest && sendToWebsocket(messageItem);
          return messageItem;
        }
        function tryAddRequest(messageItem) {
          messageItem.displayState = getDisplayState(messageItem.state);
          var media = messageItem && messageItem.digitalAssets && messageItem.digitalAssets[0];
          if (media){
            addSrc($sce, security, media);
          }
          sendToWebsocket(messageItem);
        }
        function tryRemove(message) {
          message.state = 'delete';
          message.displayState = 'deleting...';
          transportBroker.delete(message.messageId, chatId, message.digitalAssets);
        }
        function messageAsRead(message) {
          if (message.onImageLoad && message.onImageLoad.then) {
            console.log('messageAsRead: image message, waiting for onImageLoad');
            window.message = message;
            message.onImageLoad.then(function(){
              console.log('----- image message read on image load!');
              markMessageAsRead(message);
            }).catch(function(){
              console.log('----- image message read FAILED :(');
            });
            return;
          }
          if (message.state === 'queued' || message.state === 'delivered' || message.state === 'sent') {
            markMessageAsRead(message);
            //clearCache();
          }
        }
        function markMessageAsRead(message) {
          message.timestamp = (new Date()).getTime();
          message.state = 'read';
          message.displayState = getDisplayState('read');
          transportBroker.read(message.messageId, chatId);
        }
        function sendToWebsocket(messageItem) {
          transportBroker.send(messageItem.text, messageItem.messageId, chatId, cleanAssetWS(messageItem.digitalAssets && messageItem.digitalAssets[0]), messageItem.location);
        }
        function onChange(pageScope, callback) {
          pageScope && pageScope.$on('msg-collection-change-' + chatId, function(event, data){
            callback && callback(data);
          });
        }

        return instance;
      }
    };
    function cleanAssetWS(asset) {
      if (!asset) return;
      
      var cleaned = {};
      Object.keys(asset).filter(function(key){
        return key !== 'base64' && key !== 'src' && key !== 'srcNatural';
      }).forEach(function(key){
        cleaned[key] = asset[key];
      });
      return cleaned;
      //return _.props(['id', 'type'])(asset);
    }
    function normalizeItem(myProfileId, participants, $sceDelegate, $sce, security, $q) {
      return function(data){
        var textHtml = data.textHtml
            || data.location && data.location.latitude && $sceDelegate.trustAs('html', utils.cleanUpMessage(getLocationMap(data.location)))
            || data.text && getTrustedHtml($sceDelegate, data.text);
        var normalized = {
          messageId: data.messageId,
          textHtml: textHtml,
          text: data.text,
          from: data.from,
          isMine:  data.from === myProfileId || data.isMine || false,
          state: data.state,
          displayState: getDisplayState(data.state),
          participant: data.participant || getParticipant(participants, data.from),
          timestamp: data.timestamp || (new Date).getTime(),
          location: getLocationSrc(data.location),
          digitalAssets: data.digitalAssets && data.digitalAssets.map(_.curry(addSrc)($sce, security)) || [],
          screenCaptureNotification: data.screenCaptureNotification
        };
        //console.log('Normalized from-to ', data, normalized);
        var imageAsset = normalized.digitalAssets && normalized.digitalAssets[0] && normalized.digitalAssets[0].type === 'image' && normalized.digitalAssets[0];
        if (!normalized.isMine && (normalized.state === 'queued' || normalized.state === 'delivered') && imageAsset) {
          console.log('MC: found unread image message!');
          var imgDeferred = $q.defer();
          normalized.state = 'downloading';
          normalized.displayState = getDisplayState('downloading');
          imageAsset.imageControl = {};
          normalized.onImageLoad = imgDeferred.promise;
          imageAsset.imageControl.onReady = function(imageLoadPromise){
            console.log('MC: subscribe');
            imageLoadPromise.then(function(data){
              // send "read" ack here.
              console.log('MC: img loaded! ' + data);
              imgDeferred.resolve('OK');
              imageAsset.state = 'downloaded';
            }).catch(function(err){
              // show retry button:
              console.log('MC: img failed :( ' + err);
              normalized.state = 'failed';
              normalized.displayState = getDisplayState('failed');
              imageAsset.state = 'failed-download';
            });
          }
        }

        return normalized;
      }
    }
    function getParticipant(participants, id){
      id = id && id.profileId || id;
      return participants && _.find(_.where({profileId: id}))(participants);
    }
    function getDisplayState(state) {
      return state === 'sent' && 'sending' || state === 'uploading' && 'uploading...' || (state === 'queued' ? 'sent' : state);
    }
    function getLocationSrc(location) {
      if (location && location.latitude) {
        location.src = 'http://maps.google.com/maps/api/staticmap?center={latitude},{longitude}&markers=size:normal%7C{latitude},{longitude}&zoom=18&size=640x640&sensor=true'
            .replace(/\{latitude\}/g, location.latitude)
            .replace(/\{longitude\}/g, location.longitude);
        //location.src = $sce.trustAsResourceUrl(location.src);
      }
      return location;
    }
    function addSrc($sce, security, asset) {
      if ((asset.type === 'video' || asset.type === 'audio') && asset.id) {
        asset.src = getTrustedUrl(asset.id, $sce, security);
        asset.srcNatural = getUntrustedUrl(asset.id, security);
      }
      return asset;
    }
    function getTrustedUrl(mediaId, $sce, security) {
      var mediaUrl = config.getMediaUrl(mediaId);
      return $sce.trustAsResourceUrl(mediaUrl + security.bewit(mediaUrl, 6*600*1000));
    }
    function getUntrustedUrl(mediaId, security) {
      var mediaUrl = config.getMediaUrl(mediaId);
      return mediaUrl + security.bewit(mediaUrl, 6*600*1000);
    }
    function getTrustedHtml($sceDelegate, text) {
      return $sceDelegate.trustAs('html', utils.cleanUpMessage(text))
    }
    function getLocationMap(location) {
      return 'http://maps.google.com/maps?q={lat},{long}'.replace('{lat}', location.latitude).replace('{long}', location.longitude);
    }
    var _getAssetIds = _.map(_.compose(_.head, function(a){return a || [{id:0}]}, _.prop('digitalAssets')));
    function checkAssets(item, items) {
      var digitalId = item && item.digitalAssets && item.digitalAssets[0] && item.digitalAssets[0].id;
      return !(
        digitalId && items && items.length &&
        _.compose(_.find(_.where({id: digitalId})), _getAssetIds)(items)
      );
    }

    return messageCollection;
  });
}(define, angular));