
(function (define) {
  'use strict';

  define(['config', 'utils'], function (config, utils) {

    var _ = utils._,
      PUB_URL = '/pub/{profileId}',
      SUB_URL = '/sub/{profileId}/+',
      MQTT_QOS = 1,
      reconnectPeriod = config.transport.RECONNECT_PERIOD_MS,
      RECONNECT_PERIOD_OFFLINE = 60000,
      TRANSPORT_EVENT_NAME_SUB = 'transport-sub-msg',
      TRANSPORT_EVENT_NAME_PUB = 'transport-pub-msg'
      //serverUrl = config.server.mqtt;

    var mowsTransport = function ($rootScope, $q, userProfile, security, baseData) {

      var client, profileId, pubUrl, subUrl, isConnected = false, isReady = false;

      //var mows  = require('mows');

      $rootScope.$on(TRANSPORT_EVENT_NAME_PUB, onPub);
      $rootScope.$on('logout', onLogout);
      $rootScope.$on('app-offline', onPause);
      $rootScope.$on('app-pause', onPause);
      $rootScope.$on('app-online', onResume);
      $rootScope.$on('app-resume', onResume);

      function onPause() {
        client && client.options && (client.options.reconnectPeriod = RECONNECT_PERIOD_OFFLINE);
      }
      function onResume() {
        client && client.options && (client.options.reconnectPeriod = reconnectPeriod);
        client && client._reconnect && !client.connected && client._reconnect();
      }

      function init() {
        profileId = userProfile.getProfileId();
        pubUrl = PUB_URL.replace('{profileId}', profileId);
        subUrl = SUB_URL.replace('{profileId}', profileId);

        var pswd = security.getSignature();
        console.log("mqtt: init client " + config.server.mqtt + ', profileId: ' + profileId + ', password: ' + pswd);
        client = mows.createClient(config.server.mqttPort, config.server.mqtt, {
          username: profileId,
          password: pswd,
          keepalive: config.transport.MQTT_KEEPALIVE_SEC,
          reconnectPeriod: reconnectPeriod,
          // if clean is false then user will receive messages created while he was offline:
          clientId: profileId,
          clean: false
        });
        window.mowsClient = client;

        client.on('message', function (topic, message) {
          onMessageArrived(message);
        });
        client.on('connect', onConnect);
        client.on('close', onConnectionLost);
        client.on('reconnect', onReconnect);
        client.on('error', onError);
        isReady = true;
      }
      function tm(){
        return moment().format('hh:mm:ss') + ' ';
      }
      function onConnect() {
        console.log(tm() + 'mqtt: onConnect.');
        isConnected = true;
        // We want to resubscribe even for not clean session in case server got restarted:
        client && client.subscribe(subUrl, {qos: MQTT_QOS});
      }
      function onReconnect() {
        console.log(tm() + 'mqtt: onReconnect.');
        baseData.ping();
      }
      function onError(err) {
        console.log(tm() + 'mqtt: onError.', err);
      }
      function onLogout() {
        console.log('mqtt: end.');
        client && (
          client.end(),
          client.inflight.puback = {},
          client.queue = [],
          client = null
        );
      }
      function moveInfliteToQueue(){
        if (!client) return;
        var puback = client.inflight.puback;
        Object.keys(puback).forEach(function(id){
          client.queue.push(puback[id]);
          delete puback[id];
          console.log('--- moveInfliteToQueue: ' + id);
        });
      }
      function onConnectionLost() {
        console.log(tm() + 'onConnectionLost. ');
        isConnected = false;
        moveInfliteToQueue();
      }
      function onMessageArrived(message) {
        var msg = JSON.parse(message);
        console.log('mqtt <<<<<   onMessageArrived: ' + dump(msg), msg);
        $rootScope.$broadcast(TRANSPORT_EVENT_NAME_SUB, msg);
      }
      function onPub(e, data) {
        if (isReady){
          send(data);
        }
      }
      function send(data) {
        var msgString = JSON.stringify(data);
        console.log('mqtt >>>>>   sending: ' + dump(data));
        client.publish(pubUrl, msgString, {qos: MQTT_QOS, retain: false}, function(){
          console.log('mqtt ACK: published ' + dump(data));
        });
      }
      function dump(msg){
        return '[' + msg.scope + ':' + msg.type + (msg.payload && msg.payload.text && (':"' + msg.payload.text + '"') || '') + ']'; // + ' \n       - ' + JSON.stringify(msg) + '\n';
      }

      return {
        init: init,
        send: send
      }
    };

    return mowsTransport;
  });
}(define));