
(function (define, Paho) {
  'use strict';

  define(['config', 'utils'], function (config, utils) {

    var _ = utils._,
      PUB_URL = '/pub/{profileId}',
      SUB_URL = '/sub/{profileId}/+',
      MQTT_QOS = 1,
      serverUrl = 'api.test.gokismet.com';

    var pahoTransport = function ($rootScope, $q, userProfile, security, appState) {

      var client, profileId, pubUrl, subUrl, isConnected = false, isReady = false,
        reconnectTimer;

      $rootScope.$on(config.event.TRANSPORT_PUB, onPub);
      $rootScope.$on('logout', onLogout);
      $rootScope.$on('app-offline', onPause);
      $rootScope.$on('app-pause', onPause);
      $rootScope.$on('app-online', onResume);
      $rootScope.$on('app-resume', onResume);

      function init(isReconnect, options) {
        options = options || {};
        if (!isReconnect){
          profileId = userProfile.getProfileId();
          pubUrl = PUB_URL.replace('{profileId}', profileId);
          subUrl = SUB_URL.replace('{profileId}', profileId);

          client = new Paho.MQTT.Client(serverUrl, Number(config.server.mqttPort), '/', 'clientid-' + profileId);
          client.onConnectionLost = onConnectionLost;
          client.onMessageArrived = onMessageArrived;
          client.onMessageDelivered = onMessageDelivered;
          client.startTrace();
        }
        var connectOpts = angular.extend({
          userName: profileId,
          password: security.getSignature(),
          useSSL: true,
          onSuccess:onConnect,
          onFailure:onFailure,
          cleanSession: false,
          timeout: 10
        }, options);

        debug("mqtt: trying to connect... " + serverUrl, connectOpts);
        client.connect(connectOpts);
        isReady = true;
      }
      function onLogout() {
        debug('mqtt: end.');
        // TODO: implement

        //client && (
        //  client.end(),
        //  client.inflight.puback = {},
        //  client.queue = [],
        //  client = null
        //);
      }

      // called when the client connects
      function onConnect() {
        isConnected = true;
        // Once a connection has been made, make a subscription and send a message.
        debug(moment().format('hh:mm:ss') + " mqtt: onConnect");
        client.subscribe(subUrl, {
          qos: MQTT_QOS
        });
      }

      // called when the client loses its connection
      function onConnectionLost(responseObject) {
        client.stopTrace();
        debug(moment().format('hh:mm:ss') + ' Trace: ', client.getTraceLog());
        isConnected = false;
        if (responseObject.errorCode !== 0) {
          debug(moment().format('hh:mm:ss') + " mqtt: onConnectionLost: " + responseObject.errorMessage);
        }
        if (!appState.get('isBackground') && !appState.get('isOffline')) {
          reconnect();
        }
      }
      function reconnect() {
        init(true, {cleanSession: false});
      }
      function setReconnectTimer() {
        reconnectTimer = setInterval(function(){
          reconnect();
        }, 1000);
      }
      function onFailure(err) {
        //TODO: implement
        debug(moment().format('hh:mm:ss') + ' mqtt: onFailure: ...', err);
        //setTimeout(function(){init(true, {cleanSession: false})}, 5000);
      }
      function onPause() {
        client && client.options && (client.options.reconnectPeriod = RECONNECT_PERIOD_OFFLINE);
      }
      function onResume() {
        if (!isConnected) {
          init(true, {cleanSession: false});
        }
      }

      // called when a message arrives
      // @param <Paho.MQTT.Message>
      function onMessageArrived(message) {
        debug('mqtt <<<<<   onMessageArrived: ' + message.payloadString);
        var msg = JSON.parse(message.payloadString);
        $rootScope.$broadcast(config.event.TRANSPORT_SUB, msg);
      }

      // @param <Paho.MQTT.Message>
      function onMessageDelivered(message) {
        debug('mqtt <<<<<   onMessageDelivered: ' + message.payloadString);
        var msg = JSON.parse(message.payloadString);
        $rootScope.$broadcast(config.event.TRANSPORT_PUB_ACK, msg);
      }
      function onPub(e, data) {
        // TODO: add to our buffer (by subscribing to config.event.TRANSPORT_PUB event)
        // TODO: remove from our buffer on config.event.TRANSPORT_PUB_ACK which is a success.
        if (isConnected){
          send(data);
        } else {
          $rootScope.$broadcast(config.event.TRANSPORT_PUB_REJECTED, data);
        }
      }

      // Send to MQTT:
      function send(data) {
        var msgString = JSON.stringify(data),
          message = new Paho.MQTT.Message(msgString);
        debug('mqtt >>>>>   sending: ' + message.payloadString);
        message.destinationName = pubUrl;
        client.send(message);
      }

      function debug(txt, obj) {
        console.log(txt, obj);
        return;
        var debugEl = document.getElementById('debug');
        debugEl && (debugEl.innerHTML += '<br />' + txt + (obj && ', ' + JSON.stringify(obj) || ''));
      }

      return {
        init: init,
        send: send,
        end: onLogout,
        debug: debug
      }
    };

    return pahoTransport;
  });
}(define, Paho));