(function (define) {
  'use strict';

  define(['config', 'utils'], function (config, utils) {

    var _ = utils._,
      TRANSPORT_EVENT_NAME_SUB = 'transport-sub-msg',
      TRANSPORT_EVENT_NAME_PUB = 'transport-pub-msg';

    var transportBroker = function ($rootScope, $q, userProfile, localNotification, $location, settingsData, chatsCollection, chatsArchivedData) {

      var myProfileId = userProfile.getProfileId,
        getUserData = userProfile.getFull;

      $rootScope.$on(TRANSPORT_EVENT_NAME_SUB, onMessage);

      function sendMsg(messageTxt, messageId, chatId, digitalAsset, location) {
        //console.log('[transportBroker.sendMsg] ', arguments);
        var deferred = $q.defer(),
          payload = {};
        if (messageTxt) payload.text = messageTxt;
        if (digitalAsset) {
          //payload.digitalAssetId = digitalAsset.id;
          //payload.digitalAssetType = digitalAsset.type;
          payload.digitalAsset = digitalAsset;
        }
        if (location) {
          payload.location = location;
        }
        firePub(messageId, chatId, 'message', 'sent', payload, 'profileId');
        deferred.resolve('sent');
        return deferred.promise;
      }
      function deleteMsg(messageId, chatId, digitalAssets) {
        firePub(messageId, chatId, 'message', 'delete', {digitalAsset: (digitalAssets && digitalAssets[0] && _.pick(['id','type'], digitalAssets[0]))}, 'from');
      }
      function readMsg(messageId, chatId) {
        firePub(messageId, chatId, 'message', 'read', null, 'from');
      }
      function deliverMsg(messageId, chatId) {
        firePub(messageId, chatId, 'message', 'delivered', null, 'from');
      }
      function typing(chatId) {
        firePub(null, chatId, 'chat', 'typing', null, 'from');
      }
      function joinChat(chatId) {
        firePub(null, chatId, 'chat', 'joined', null, 'from');
      }
      function wipeChat(chatId) {
        firePub(null, chatId, 'chat', 'wipe', null, 'from');
      }
      function firePub(messageId, chatId, scope, type, payload, from) {
        var userData = getUserData();
        var obj = {
          chatId: chatId,
          scope: scope,
          from: from && userData.profileId || {
            profileId: userData.profileId,
            firstName: userData.firstName
          },
          type: type,
          timestamp: (new Date()).getTime(),
          messageId: messageId,
          payload: payload
        };
        //console.log('[transportBroker.firePub] obj: ' + JSON.stringify(obj));
        $rootScope.$broadcast(TRANSPORT_EVENT_NAME_PUB, obj);
      }
      function fire(eventName, payload){
        $rootScope.$broadcast(eventName, payload);
      }


      function onMessage(event, data) {
        //console.log('Broker.onMessage: ' + data.scope + ':' + data.type);
        if (data.type === 'typing') {
          data.scope = 'message';
        }
        switch (data.scope) {
          case 'chat':
            onChatMessage(data.type, data.chatId, data);
            break;

          case 'message':
            onMessageMessage(data.type, data.chatId, data);
            break;
        }
      }
      function onChatMessage(type, chatId, msg) {
        switch (type) {
          case 'new':
            fire(config.event.NEW_MATCH, {
              participants: [
                {imageId: msg.fromImageId, firstName: msg.fromFirstName, profileId: msg.from, chatId: msg.chatId}
              ]
            });
            settingsData.get().then(function(settings){
              localNotification.add({
                id:         'kismet-new-match',
                message:    settings && !settings.privatePush ? 'You’ve matched with ' + msg.fromFirstName : 'You have a new match!',
                json:       {profileId: msg.from}
              }, function(data){
                $location.path('/app/profiles/' + data.profileId);
              });
            });
            break;
          case 'wipe':
            fire(config.event.CHAT_WIPE, {chatId: chatId, message: msg});
            fire(config.event.CHAT_WIPE + '-' + chatId, msg);
            break;
        }
      }
      function onMessageMessage(type, chatId, msg) {
        switch (type) {
          case 'queued':
            // acknowledgement:
            if (msg.from === myProfileId()) {
              fire(config.event.CHAT_MESSAGE_SENT + '-' + msg.chatId, msg);
            }
            // new message:
            else {
              var obj = {
                chatId: msg.chatId,
                messageId: msg.messageId,
                text: msg.payload && msg.payload.text,
                screenCaptureNotification: msg.payload && msg.payload.screenCaptureNotification,
                from: msg.from && msg.from.profileId && msg.from || {profileId: msg.from},
                location: msg.payload.location
              };
              if (msg.payload.digitalAsset) {
                obj.digitalAssets = [msg.payload.digitalAsset];
              }
              //console.log('MessageBroker: new chat message for ' + msg.chatId);
              fire(config.event.CHAT_MESSAGE_NEW, obj);
              fire(config.event.CHAT_MESSAGE_NEW + '-' + msg.chatId, obj);
              settingsData.get().then(function(settings){
                var participant = getParticipant(msg.chatId),
                  name = participant && participant.firstName;
                localNotification.add({
                  id:         'kismet-new-chat-message',
                  message:    name + ': ' + (settings && !settings.privatePush ? (obj.text || 'New message') : 'New message'),
                  json:       {chatId: chatId}
                }, function(data){
                  $location.path('/app/chat-room/' + data.chatId);
                });
              });
            }
            break;

          case 'delete':
            type = 'deleted';
          case 'deleted':
            fire(config.event.CHAT_MESSAGE_DELETED, {chatId: chatId, message: msg});
          case 'typing':
          case 'wipe':
          case 'delivered':
          case 'read':
            fire(config.event['CHAT_MESSAGE_' + type.toUpperCase()] + '-' + msg.chatId, msg);
            break;

          default:
            console.log('WARNING: transportBroker: unknown message type: ' + type);
        }
      }
      function getParticipant(chatId) {
        return (chatsCollection.get(chatId) || chatsArchivedData.get(chatId) || {}).participant;
      }

      return {
        send: sendMsg,
        delete: deleteMsg,
        read: readMsg,
        delivered: deliverMsg,
        typing: typing,
        //joinChat: joinChat,
        wipeChat: wipeChat
      };
    };

    return transportBroker;
  });
}(define));