/***

  Login info:
  {
    accountInfo: {}
    firstTimeRegistering: false
    hasPortableAccount: true
    key: "6c7f992082aeb5d24bb7d80c145707deca4b5e42"
    profileComplete: true
    profileId: "a54aef35-796a-4b9c-9ea9-f7928207094a"
  }

 Profile info:
 {
    "firstName": "Lemonchella",
    "age": 23,
    "gender": "f",
    "genderPreferenceMale": true,
    "genderPreferenceFemale": true,
    "blurbs": [{
      "title": "",
      "text": "I like lemons and I am humgry"
    }],
    "profileImages": [{
      "imageId": "120a9710-50c9-11e4-8ee6-3d1a83d23c21",
      "primary": true
    }]
  }

 In LocalStorage we store:
 - "profileId"
 - "<profileId>-loginData"
 - "<profileId>-userProfile"

***/

(function (define) {
  'use strict';

  define(['config', 'validator', 'utils'], function (config, validator, utils) {

    var _ = utils._,
      maybe = utils.maybe({}),
      URL = config.apiUrls;

    var userProfile = function ($q, $rootScope, baseData, security) {

      var _profileId, _loginInfo, _userInfo,
        deferreds = {
          loginInfo: $q.defer(),
          userInfo: $q.defer()
        },loginDeferred;

      // Should be used only for login button action:
      var login = function(data, isSignup){
        if (!loginDeferred){
          loginDeferred = $q.defer();

          var url = isSignup ? URL.SIGNUP : URL.LOGIN;

          baseData
            .post(url, {
                countryCode: data.countryCode,
                phoneNumber: data.username,
                pin: data.password
            }, null, 'no-ping-check')
            .then(function(data) {
              //Response: { key, profileId, firstTimeRegistering, profileComplete }

              _profileId = data.profileId;
              _loginInfo = data;
              localSet('profileId', _profileId);
              localSet('loginData', data, _profileId);
              loginDeferred.resolve(data);
              deferreds.loginInfo.resolve(data);
            })
            .catch(function(error) {
              loginDeferred.reject(error.message);
              loginDeferred = null;
            });
        } else {
          //console.log('WARNING: userProfile do not login since the deferred object was already created. No cleanup applied?');
        }
        return loginDeferred.promise;
      };

      var hasInputErrors = function(userData, isSignup) {
        var isError = false, errors = {};
        if (!userData.username){
          errors.email = 'FORM_ERROR_USER_ID_REQUIRED';
          isError = true;
        } else if (!validator.username.test(userData.username)) {
          errors.email = 'FORM_ERROR_USER_ID_INVALID';
          isError = true;
        }
        if (!isSignup) {
          if (!userData.password) {
            errors.password = 'FORM_ERROR_PASSWORD_REQUIRED';
            isError = true;
          } else if (userData.password.length != 5) {
            errors.password = 'FORM_ERROR_PASSWORD_INVALID';
            isError = true;
          }
        }
        return isError ? errors : false;
      };

      var hasProfileErrors = function(data) {
        var isError = false, errors = {};
        if (!data.firstName) {
          errors.firstName = 'PROFILE_ERROR_FIRST_NAME_REQUIRED';
          isError = true;
        }
        if (!data.age) {
          errors.age = 'PROFILE_ERROR_AGE_REQUIRED';
          isError = true;
        }
        if (!data.gender) {
          errors.gender = 'PROFILE_ERROR_GENDER_REQUIRED';
          isError = true;
        }
        if (!data.genderPreferenceFemale && !data.genderPreferenceMale) {
          errors.genderPreference = 'PROFILE_ERROR_GENDER_PREFERENCE_REQUIRED';
          isError = true;
        }
        if (data.uploading) {
          errors.imageId = 'PROFILE_ERROR_PHOTO_UPLOADING';
          isError = true;
        }
        if (!data.imageId && !data.uploading) {
          errors.imageId = 'PROFILE_ERROR_PHOTO_REQUIRED';
          isError = true;
        }
        return isError ? errors : false;
      };

      var loadProfileId = function() {
        var deferred = $q.defer();
        if (checkLocalStorageLogin()){
          deferred.resolve(_profileId);
        } else {
          deferred.reject('Not logged in yet.');
        }

        return deferred.promise;
      };

      var loadInfo = function(force) {
        if (!force && checkLocalStorageItem('userProfile', deferreds.userInfo, _profileId)) {
          //console.log('userProfile loaded from localStorage: ' + JSON.stringify(localGet('userProfile', _profileId)));
          return deferreds.userInfo.promise;
        }

        return deferreds.loginInfo.promise.then(function(loginData){
          var url = URL.LOAD_PROFILE.replace('{profileId}', loginData.profileId);
          return baseData.get(url).then(function(data){
            //{firstName:"", age:18, gender:"", genderPreferenceMale: false, genderPreferenceFemale:false, blurbs: [{title: "", text:""}], profileImages:[{imageId:''}]},

            _userInfo = data;
            localSet('userProfile', data, _profileId);
            deferreds.userInfo.resolve(data);
            return data;
          });
        })
      };

      function checkLocalStorageLogin() {
        var profileId = localGet('profileId');
        if (profileId) {
          //console.log('+++ checkLocalStorageLogin ' + profileId);
          _profileId = profileId;
          _loginInfo = localGet('loginData', _profileId);
          _userInfo = localGet('userProfile', _profileId);
          loginDeferred = $q.defer();
          loginDeferred.resolve(_loginInfo);
          deferreds.loginInfo.resolve(_loginInfo);
          return true;
        }
        return false;
      }

      function checkLocalStorageItem(name, deferred, prefix) {
        var data = localGet(name, prefix);
        if (data) {
          deferred.resolve(data);
          return true;
        }
        return false;
      }

      var save = function(data){
        var deferred = $q.defer(),
          errors = hasProfileErrors(data);

        if (errors) {
          deferred.reject(errors);
        } else {
          saveToServer(data).then(function(data) {
            deferreds.userInfo = $q.defer();
            deferreds.userInfo.resolve(data);
            localSet('userProfile', data, _profileId);
            _userInfo = data;

            var loginData = localGet('loginData', _profileId);
            loginData.profileComplete = true;
            localSet('loginData', loginData, _profileId);
            deferreds.loginInfo = $q.defer();
            deferreds.loginInfo.resolve(loginData);

            deferred.resolve(data);
          });
        }
        return deferred.promise;
      };

      var cleanUserData = function() {
        deferreds.loginInfo = $q.defer();
        deferreds.userInfo = $q.defer();
        loginDeferred = null;
        localRemove('profileId');
        localCleanup(_profileId);
        _profileId = null;
        _userInfo = null;
        //console.log('userProfile: broadcasting logout ...');
        $rootScope.$broadcast('logout');
      };

      var registerDevice = function(deviceInfo, langInfo) {
        var opt = {
          'deviceId': deviceInfo.deviceID,
          'preferredLanguage': langInfo.lang,
          'localeName': langInfo.locale,
          'userPlatform': deviceInfo.userPlatform,
          'userPlatformVersion': deviceInfo.userPlatformVersion,
          'userDeviceModel': deviceInfo.userDeviceModel
        };
        return baseData.post(config.apiUrls.REGISTER_DEVICE.replace('{profileId}', _profileId), opt);
      };

      function saveToServer(data) {
        var url = URL.SAVE_PROFILE.replace('{profileId}', _profileId);
        return baseData.post(url, data);
      }


      /** LOCAL STORAGE FUNCTIONS **/

      function localSet(name, obj, prefix) {
        prefix = prefix && prefix + '-' || '';
        localStorage.setItem(prefix + name, JSON.stringify(obj));
      }

      function localGet(name, prefix) {
        prefix = prefix && prefix + '-' || '';
        return JSON.parse(localStorage.getItem(prefix + name));
      }

      function localRemove(name, prefix) {
        prefix = prefix && prefix + '-' || '';
        localRemoveItem(prefix + name);
      }

      function localRemoveItem(key) {
        localStorage.removeItem(key);
      }

      function localCleanup(prefix) {
        Object.keys(localStorage)
          .filter(function(item){ return item.search(prefix) === 0;})
          .forEach(localRemoveItem);

        //var re = new RegExp('^' + prefix);
        //_.map(removeItem, _.filter(_.match(re), _.keys(localStorage)))
      }


      /** PUBLIC API **/

      return {
        // sync:
        getProfileId: function() { return _profileId; },
        getFull: function() {
          if (!_userInfo) return;

          _userInfo.imageId = _userInfo.profileImages[0] && _userInfo.profileImages[0].imageId;
          return _.mixin(
            _.pick(['profileId','profileComplete','firstTimeRegistering'], _loginInfo),
            _.pick(['firstName','age','gender','imageId','genderPreferenceMale','genderPreferenceFemale'], _userInfo)
          );
        },

        // async:
        getLoginInfo: function() {
          checkLocalStorageLogin();
          return deferreds.loginInfo.promise;
        },
        getUserInfo: function() {
          return deferreds.userInfo.promise;
        },

        // other:
        login: login,
        hasInputErrors: hasInputErrors,
        logout: cleanUserData,
        isLoggedIn: checkLocalStorageLogin, // TODO: separate the check action from local storage sync actions!
        loadProfileId: loadProfileId,
        loadInfo: loadInfo,
        getImageId: function() {
          return deferreds.userInfo.promise.then(_.compose(_.prop('imageId'), maybe, _.head, _.prop('profileImages')))
        },
        save: save,
        registerDevice: registerDevice
      }
    };

    return userProfile;
  })
}(define));
