/**
 * Different dialogs.
 */

(function (define) {
  'use strict';

  define(['config'], function (config) {

    var dialogs = function Dialogs($q, $translate, $ionicPopup) {

      function blockUserDialog(){
        var deferred = $q.defer();

        navigator.vibrate(config.popups.confirmVibrateTime);
        $q.all({okText: $translate('POPUP_OK'), cancelText: $translate('POPUP_CANCEL'), title: $translate('POPUP_BLOCK_TITLE'), text: $translate('POPUP_BLOCK_TEXT')})
          .then(function(translation){
            var confirmPopup = $ionicPopup.confirm({
              title: translation.title,
              template: translation.text,
              okText: translation.okText,
              cancelText: translation.cancelText
            });
            confirmPopup.then(function(res) {
              if(res) {
                deferred.resolve('OK');
                //blockUserForever();
              } else {
                deferred.reject('CANCEL');
              }
            });
          })
          .catch(function(){
            var confirmPopup = $ionicPopup.confirm({
              title: 'POPUP_WIPE_TITLE',
              template: 'POPUP_WIPE_TEXT',
              okText: 'POPUP_OK',
              cancelText: 'POPUP_CANCEL'
            });
            confirmPopup.then(function(res) {
              if(res) {
                deferred.resolve('OK');
                //blockUserForever();
              } else {
                deferred.reject('CANCEL');
              }
            });
          });

        return deferred.promise;
      }

      function wipeMessage() {
        var deferred = $q.defer();
        $q.all({okText: $translate('POPUP_OK'), cancelText: $translate('POPUP_CANCEL'), title: $translate('POPUP_WIPE_MESSAGE_TITLE'), text: $translate('POPUP_WIPE_MESSAGE_TEXT')})
          .then(function(translation){
            var confirmPopup = $ionicPopup.confirm({
              title: translation.title,
              template: translation.text,
              okText: translation.okText,
              cancelText: translation.cancelText
            });
            confirmPopup.then(function(res) {
              if(res) {
                deferred.resolve('OK');
                //$scope.removeMessage(message);
              } else {
                deferred.reject('CANCEL');
              }
            });
          })
          .catch(function(){
            var confirmPopup = $ionicPopup.confirm({
              title: 'POPUP_WIPE_MESSAGE_TITLE',
              template: 'POPUP_WIPE_MESSAGE_TEXT',
              okText: 'POPUP_OK',
              cancelText: 'POPUP_CANCEL'
            });
            confirmPopup.then(function(res) {
              if(res) {
                deferred.resolve('OK');
                //$scope.removeMessage(message);
              } else {
                deferred.reject('CANCEL');
              }
            });
          });

        return deferred.promise;
      }
      function wipeChat() {
        var deferred = $q.defer();
        navigator.vibrate(config.popups.confirmVibrateTime);
        $q.all({okText: $translate('POPUP_OK'), cancelText: $translate('POPUP_CANCEL'), title: $translate('POPUP_WIPE_TITLE'), text: $translate('POPUP_WIPE_TEXT')})
          .then(function(translation){
            var confirmPopup = $ionicPopup.confirm({
              title: translation.title,
              template: translation.text,
              okText: translation.okText,
              cancelText: translation.cancelText
            });
            confirmPopup.then(function(res) {
              if(res) {
                deferred.resolve('OK');
                //$scope.popover.hide();
                //wipeThisChat();
                //collection.clear();
              } else {
                deferred.reject('CANCEL');
              }
            });
          })
          .catch(function(){
            var confirmPopup = $ionicPopup.confirm({
              title: 'POPUP_WIPE_TITLE',
              template: 'POPUP_WIPE_TEXT',
              okText: 'POPUP_OK',
              cancelText: 'POPUP_CANCEL'
            });
            confirmPopup.then(function(res) {
              if(res) {
                deferred.resolve('OK');
                //$scope.popover.hide();
                //wipeThisChat();
                //collection.clear();
              } else {
                deferred.reject('CANCEL');
              }
            });
          });
        return deferred.promise;
      }

      return {
        blockUser: blockUserDialog,
        wipeMessage: wipeMessage,
        wipeChat: wipeChat
      };
    };

    return dialogs;
  })
}(define));