/**
 * Popover notification (e.g. for push notification to show up)
 */

(function (define) {
  'use strict';

  define(['config'], function (config) {

    var notificationPopover = function($rootScope, $q, $ionicModal, $timeout, $location, chatsCollection, chatsArchivedData, messageCollection) {

      var popovers = [], ttl,
        $scope = $rootScope.$new();

      var build = function(ttl) {
        var popover,
          scope = $scope.$new();

        scope.control = {};

        var popoverDeferred = $ionicModal.fromTemplateUrl('templates/dialogs/notification-popover.html', {
            scope: scope,
            animation: 'slide-in-left',
            viewType: 'popover'
          });

        return { popover: popoverDeferred, scope: scope, ttl: ttl};
      };

      var init = function(_ttl) {
        if (popovers.length === 0){
          ttl = _ttl;
          popovers.push(build(_ttl));
        }
      };
      var show = function(data) {
        if (popovers.length === 0){
          //console.log('WARNING: no notification popover created');
          return;
        }
        var popoverObj = popovers[0];
        popoverObj.scope.imageId = data.imageId;
        popoverObj.scope.message = data.message;
        popoverObj.scope.action = data.action;
        popoverObj.popover.then(function(popover){
          popover.show().then(function(){
            popoverObj.scope.control.refresh(data.imageId);
            navigator.vibrate(config.popups.confirmVibrateTime);
          });
          scheduleHide(popoverObj, function(){
            popoverObj.scope.control.refresh(null);
          });
        });
      };
      var hide = function() {
        popovers[0] && popovers[0].popover.then(function(p){p.hide();});
      };
      var setup = function() {
        init(config.popup.ttl);
        //subscribe();
      };
      var scheduleHide = function(popoverObj, callback) {
        popoverObj.popover.then(function(popover){
          $timeout(function(){
            popover.hide().then(function(){
              callback && callback();
            });
          }, popoverObj.ttl * 1000);
        });
      };
      var test = function() {
        $timeout(function(){
          $rootScope.$broadcast('push-message', {event: 'test', text: 'Hello', otherFirstName: 'UserA'});
        }, 3000);
      };
      var listenAndPopup = function(scope, callback, options) {
        var eventName = options && options.eventName || config.event.CHAT_MESSAGE_NEW,
          beforeCheck = options && typeof options.beforeCheck === 'function' && options.beforeCheck || function(){return true};
        scope.$on(eventName, function(event, data){
          if (beforeCheck(data) === false) {
            return;
          }
          var participant = (chatsCollection.get(data.chatId) || chatsArchivedData.get(data.chatId) || {}).participant;
          show({
            imageId: data.from.imageId || participant && participant.imageId,
            message: (data.from.firstName || participant && participant.firstName || 'New message') + ': '
              + (data.text || data.location && 'location' || data.digitalAssets && data.digitalAssets[0] && data.digitalAssets[0].type),
            action: data.chatId && (function(chatId){
              return function(){
                $location.path('/app/chat-room/' + chatId);
                hide();
              }
            }(data.chatId))
          });
          chatsCollection.clear();
          var mc = messageCollection(data.chatId, null, {isExisting:true});
          mc && mc.clear();
          // { chatId, lastMessage, lastMessageFromYou, lastMessageTimeStamp, hasNewMessage }
          chatsCollection.updateChat({
            chatId: data.chatId,
            lastMessage: data.text,
            lastMessageFromYou: false,
            hasNewMessage: true
          });
          callback && callback(data);
        });
      };

      function subscribe(){
        $rootScope.$on('push-message', function(event, data){
          show({
            title: data.event,
            imageId: data.otherImageId,
            message: data.otherFirstName + ': ' + data.text
          });
        });
      }

      return {
        setup: setup,
        init: init,
        show: show,
        hide: hide,
        test: test,
        listenAndPopup: listenAndPopup
      }
    };

    return notificationPopover;
  })
}(define));



/***
  data: {
    text, otherImageId, otherFirstName, chatId, otherProfileId,
    event: <"newMatch" | "newChatMessage" | "systemMessage">
  }
***/