(function (define) {
  'use strict';

  define(['config'], function (config) {

    var IMAGE_DOWNLOAD_TIMEOUT_SEC = 10;

    var kmImgBgDir = function ($timeout, $q) {

      //TODO: move this cache control to imageData.
      var urlCache = {};
      // since security bewit has expiration period we have to reset url cache:
      setInterval(function(){
        urlCache = {};
      }, config.ttl.FULL_IMAGE * 1000);
      function preloadImage(imageId, imageUrl, element, imageLoadDeferred){
        $timeout(function(){
          imageLoadDeferred.reject('timeout');
        }, IMAGE_DOWNLOAD_TIMEOUT_SEC * 1000);
        var tmp = angular.element('<img />');
        tmp.bind('load', function(){
          element.css('background','url(' + imageUrl + ') no-repeat center center');
          urlCache[imageId] = imageUrl;
          console.log('kmImgBg: full image preloaded');
          imageLoadDeferred.resolve('full image OK');
        });
        tmp.attr('src', imageUrl);
      }

      return {
        restrict: 'A',
        controller: function($scope, $q, imageData){
          this.getImage = function(imageId, size){
            if (!imageId) {
              var d = $q.defer();
              d.reject('no image id');
              return d.promise;
            }

            size = size || 'thumb';
            if (imageId.then) {
              return imageId.then(function(imageId) { return imageData.getImage(imageId, size); });
            }
            return imageData.getImage(imageId, size);
          };
          this.getImageUrl = function(imageId, size){
            if (imageId.then) {
              return imageId.then(function(imageId) { return imageData.getImageUrl(imageId, size); });
            }
            var deferred = $q.defer();
            deferred.resolve(imageData.getImageUrl(imageId, size));
            return deferred.promise;
          };
        },
        scope: {
          'imageId': '=',
          'imageSrc': '=',
          'size': '=',
          'type': '=',
          'control': '=',
          'ratio': '='
        },
        link: function (scope, element, attrs, ctrl) {
          //console.log('km-img-bg: ' + scope.imageId + ' ' + scope.size + ' ' + scope.type);
          var size = scope.size || 'thumb',
            hasFullImage = size === 'normal';

          if (scope.control) {
            scope.control.refresh = refresh;
            scope.control.retry = retry;
            // onReady is set by processImage to receive imageLoadDeferred.promise:
            // scope.control.onReady
          }

          element.css('background', 'url(' + (scope.imageSrc || size == 'thumb' && 'img/ionic.png' || 'img/blank.png') + ') no-repeat center center');

          if (!scope.imageId){
            return;
          }
          processImage();

          function refresh(imageId, data) {
            if (imageId) {
              element.css({'opacity':0})
              $timeout(function(){
                scope.imageId = imageId;
                processImage();
              }, 500);
            } else if (data){
              element.css('background','url(' + data + ') no-repeat center center');
            } else {
              element.css('background','url(img/ionic.png) no-repeat center center');
            }
          }
          function retry() {
            console.log('----- kmImgBg: retrying');
            processImage();
          }
          function processImage() {
            var imageLoadDeferred = $q.defer();

            if (scope.control && scope.control.onReady){
              console.log('[kmImgBg]: give onReady a promise');
              scope.control.onReady(imageLoadDeferred.promise);
            }

            //ctrl[scope.type === 'binary' ? 'getImageUrl' : 'getImage'](scope.imageId, size)

            //if (!hasFullImage || !urlCache[scope.imageId]) {
              ctrl.getImage(scope.imageId, 'thumb')
                .then(function(imageUrl){
                  element.css('background','url(' + imageUrl + ') no-repeat center center fixed');
                  imageLoadDeferred.resolve('thumb OK');
                })
                .catch(function(error){
                  //console.log('ERROR img', error);
                });
            //}

            if (hasFullImage) {

              if (urlCache[scope.imageId]) {
                preloadImage(scope.imageId, urlCache[scope.imageId], element, imageLoadDeferred);

              } else {
                ctrl.getImageUrl(scope.imageId, 'normal')
                  .then(function(imageUrl){
                    preloadImage(scope.imageId, imageUrl, element, imageLoadDeferred);
                  })
                  .catch(function(error){
                    $timeout(function(){
                      imageLoadDeferred.reject('timeout');
                    }, IMAGE_DOWNLOAD_TIMEOUT_SEC * 1000);
                  });
              }
            }
            $timeout(function(){
              element.css({'opacity':1})
            }, 500);
          }
        }
      };

    };

    return kmImgBgDir;
  });


}(define));