(function (define) {
  'use strict';

  define(['config'], function (config) {

    var kmImgDir = function () {

      //TODO: move this cache control to imageData.
      var urlCache = {};
      // since security bewit has expiration period we have to reset url cache:
      setInterval(function(){
        urlCache = {};
      }, config.ttl.FULL_IMAGE * 1000);
      function preloadImage(imageId, imageUrl, element){
        var tmp = angular.element('<img />');
        tmp.bind('load', function(){
          element.attr('src', imageUrl);
          urlCache[imageId] = imageUrl;
        });
        tmp.attr('src', imageUrl);
      }

      return {
        restrict: 'A',
        controller: function($scope, $q, imageData){
          this.getImage = function(imageId, size){
            if (!imageId) {
              var d = $q.defer();
              d.reject('no image id');
              return d.promise;
            }

            size = size || 'thumb';
            if (imageId.then) {
              return imageId.then(function(imageId) { return imageData.getImage(imageId, size); });
            }
            return imageData.getImage(imageId, size);
          };
          this.getImageUrl = function(imageId, size){
            if (imageId.then) {
              return imageId.then(function(imageId) { return imageData.getImageUrl(imageId, size); });
            }
            var deferred = $q.defer();
            deferred.resolve(imageData.getImageUrl(imageId, size));
            return deferred.promise;
          };
        },
        scope: {
          'imageId': '=',
          'imageSrc': '=',
          'size': '=',
          'type': '=',
          'control': '=',
          'ratio': '='
        },
        link: function (scope, element, attrs, ctrl) {
          //console.log('km-img: ' + scope.imageId + ' ' + scope.size + ' ' + scope.type);
          var size = scope.size || 'thumb',
            hasFullImage = size === 'normal';

          setHeight(element);
          //if (scope.imageId && urlCache[scope.imageId]) {
          //  element.attr('src', urlCache[scope.imageId]);
          //  return;
          //}
          element.attr('src', (scope.imageSrc || size == 'thumb' && 'img/ionic.png' || 'img/blank.png') );

          if (scope.control) {
            scope.control.refresh = refresh;
          }

          if (!scope.imageId){
            return;
          }
          processImage();

          scope.$on('app-resize', function(event){
            setHeight(element);
          });

          function setHeight(element){
            if (scope.ratio && scope.ratio === 'keep') return;
            var imgWidth = element[0].offsetWidth;
            if (imgWidth) {
              element.css('height', imgWidth + 'px');
            }
          }
          function refresh(imageId, data) {
            if (imageId) {
              scope.imageId = imageId;
              processImage();
              setHeight(element);
            } else if (data){
              element.attr('src', data);
            } else {
              element.attr('src', 'img/ionic.png');
            }
          }
          function processImage() {
            //ctrl[scope.type === 'binary' ? 'getImageUrl' : 'getImage'](scope.imageId, size)

            //if (!hasFullImage || !urlCache[scope.imageId]) {
              ctrl.getImage(scope.imageId, 'thumb')
                .then(function(imageUrl){
                  element.attr('src', imageUrl);
                })
                .catch(function(error){
                  //console.log('ERROR img', error);
                });
            //}

            if (hasFullImage) {

              if (urlCache[scope.imageId]) {
                preloadImage(scope.imageId, urlCache[scope.imageId], element);

              } else {
                ctrl.getImageUrl(scope.imageId, 'normal')
                  .then(function(imageUrl){
                    preloadImage(scope.imageId, imageUrl, element);
                  })
                  .catch(function(error){
                    //console.log('ERROR img', error);
                  });
              }
            }
          }
        }
      };

    };

    return kmImgDir;
  });


}(define));