(function (define) {
  'use strict';

  define(['config'], function (config) {

    var SCROLL_THROTTLE_MS = 100,
      captureOnlyScrollDown = true;

    var kmInfiniteScrollDir = function () {
      return {
        restrict: 'A',
        controller: function($scope, $rootScope, $timeout){

          function touchEnd(){
            $rootScope.$broadcast('km-scroll-touchend');
          }
          document.addEventListener('touchend', touchEnd);
          $scope.$on("$destroy", function() {
            document.removeEventListener('touchend', touchEnd);
          });

          this.init = function(element, captureEvent, distance, scrollObj){
            // el.offsetHeight + el.scrollTop = el.scrollHeight
            var el = window.el = element[0],
              prevScrollTop = 0,
              scrollDir = getScrollDir();

            element.bind('scroll', function() {
              scrollObj.top = el.scrollTop;
              $rootScope.$broadcast('km-scroll-top', scrollObj.top);
              throttle();
            });
            $scope.$on('km-scroll', function() {
              throttle('force');
            });

            function throttle(force) {
              if (throttle.throttleFlag || force) {

                getScrollDir();
                onInfiniteScroll(force);
                onPullToRequest();

                throttle.throttleFlag = false;
                setTimeout(function(){
                  throttle.throttleFlag = true;
                }, SCROLL_THROTTLE_MS);
              }
            }
            throttle.throttleFlag = true;

            function getScrollDir(){
              scrollDir = el.scrollTop - prevScrollTop > 0 && 'down' || 'up';
              prevScrollTop = el.scrollTop;
              return scrollDir;
            }

            /**
             * Pull To Request
             */
            var pullElHeight = 60,
              triggerHeight = pullElHeight,
              SCROLL_PTR_ACTIVATE = config.event.SCROLL_PTR_ACTIVATE,
              SCROLL_PTR_DEACTIVATE = config.event.SCROLL_PTR_DEACTIVATE,
              SCROLL_PTR_START = config.event.SCROLL_PTR_START,
              TIMEOUT_AFTER_START_SEC = 10,
              pullBlockEvents = false;
            //var prCapturedEvent = false;
            function onPullToRequest(){
              if (pullBlockEvents) return;
              if (el.scrollTop <= (-triggerHeight)) {
                $rootScope.$broadcast(SCROLL_PTR_ACTIVATE);
              } else {
                $rootScope.$broadcast(SCROLL_PTR_DEACTIVATE);
              }
            }
            $rootScope.$on('km-scroll-touchend', function(){
              if (pullBlockEvents) return;
              if (el.scrollTop <= (-triggerHeight)) {
                pullBlockEvents = true;
                $rootScope.$broadcast(SCROLL_PTR_START);
                $timeout(function(){
                  pullBlockEvents = false;
                }, TIMEOUT_AFTER_START_SEC * 1000);
              }
            });

            /**
             * Infinite Scroll Loading
             */
            var isCapturedEvent = false;
            function onInfiniteScroll(force){
              if (isCapturedEvent && !force) {
                if (scrollDir === 'up') {
                  isCapturedEvent = false;
                }
                return;
              }
              if (scrollDir === 'down' || !captureOnlyScrollDown || force) {
                var fromBottom = el.scrollHeight - (el.offsetHeight + el.scrollTop);
                if (fromBottom < distance) {
                  $rootScope.$broadcast(captureEvent);
                  console.log('SCROLL: event! ' + fromBottom + ' broadcasting ' + captureEvent);
                  isCapturedEvent = true;
                }
              }

            }

          };
        },
        /*scope: {
          onInfinite: '&onInfinite',
          distance: '='
        },*/
        link: function (scope, element, attrs, ctrl) {
          var distance = parseInt(attrs.distance),
            captureEvent = attrs.triggerEvent;
          ctrl.init(element, captureEvent, distance, scope.scroll);
        }
      };
    };

    return kmInfiniteScrollDir;
  });


}(define));