(function (define) {
  'use strict';

  define(['config'], function (config) {

    var kmImgDir = function ($ionicBind) {

      //TODO: move this cache control to imageData.
      var urlCache = {};
      // since security bewit has expiration period we have to reset url cache:
      setInterval(function(){
        urlCache = {};
      }, config.ttl.FULL_IMAGE * 1000);
      function preloadImage(imageId, imageUrl, element){
        var tmp = angular.element('<img />');
        tmp.bind('load', function(){
          var eleImgId = element.attr('imageId');
          if (imageUrl.indexOf(eleImgId) > -1)
          {
            element.attr('src', imageUrl);
          }
          urlCache[imageId] = imageUrl;
        });
        tmp.attr('src', imageUrl);
      }

      return {
        restrict: 'A',
        controller: function($scope, $q, imageData){
          this.getImage = function(imageId, size){
            if (!imageId) {
              var d = $q.defer();
              d.reject('no image id');
              return d.promise;
            }

            size = size || 'thumb';
            if (imageId.then) {
              return imageId.then(function(imageId) { return imageData.getImage(imageId, size); });
            }
            return imageData.getImage(imageId, size);
          };
          this.getImageUrl = function(imageId, size){
            if (imageId.then) {
              return imageId.then(function(imageId) { return imageData.getImageUrl(imageId, size); });
            }
            var deferred = $q.defer();
            deferred.resolve(imageData.getImageUrl(imageId, size));
            return deferred.promise;
          };

          this.getImageFromCacheMethod = function(imageId, size)
          {
            return imageData.getImageFromCacheMethod(imageId,size);
          };

        },
        priority: 99,
        link: function (scope, element, attrs, ctrl) {
          scope.imageId = scope.img;

          setHeight(element);
          element.attr('src', 'img/ionic.png');

          scope.refresh = refresh;
          scope.$watch(function(scope){
            return scope.user && scope.user.img;
          }, function(newValue) {
            element.attr('src', 'img/ionic.png');
            scope.refresh(newValue);
          });
          if (scope.control) {
            scope.control.refresh = refresh;
          }

          if (!scope.imageId){
            return;
          }
          processImage();

          scope.$on('app-resize', function(event){
            setHeight(element);
          });

          function setHeight(element){
            element.css({
              'width': scope.contentHeight - 255 + 'px',
              'height': scope.contentHeight - 255 + 'px',
              'max-height': scope.contentWidth + 'px',
              'max-width': scope.contentWidth + 'px'
            });
          }
          scope.refresh = refresh;
          function refresh(imageId, data) {
            if (imageId) {
              scope.imageId = imageId;
              processImage();
              setHeight(element);
            } else if (data){
              element.attr('src', data);
            } else {
              processImage();
            }
          }
          function processImage() {
            var imageId = scope.imageId;
            ctrl.getImage(scope.imageId, 'thumb')
                .then(function(imageUrl){
                  ctrl.getImageFromCacheMethod(scope.imageId, 'thumb')
                      .then(function(imageUrl2){
                        if (scope.imageId === imageId)
                        {
                          element.attr('src', imageUrl2);
                        }
                      })
                })
                .catch(function(error){
                  console.log('ERROR img', error);
                });

            if (urlCache[scope.imageId]) {
              preloadImage(scope.imageId, urlCache[scope.imageId], element);

            } else {
              ctrl.getImageUrl(scope.imageId, 'normal')
                  .then(function(imageUrl){
                    preloadImage(scope.imageId, imageUrl, element);
                  })
                  .catch(function(error){
                    console.log('ERROR img', error);
                  });
            }
          }
        }
      };

    };

    return kmImgDir;
  });


}(define));