(function (define) {
  'use strict';

  define(['config', 'utils'], function (config, utils) {

    var _ = utils._,
      pageSize = config.data.matchesViewPageSize,
      fillDrawer = _.compose(_.take(5), _.reverse, _.filter(function(a){ return a.participant.profileId != 'kismet_support_profile'})),
      firstInit = true,
      locationError = false,
      currentIndexSaved;

    var MatchesCtrl = function ($scope, $rootScope, matchesData, $timeout, $ionicScrollDelegate, chatsCollection, settingsData, notificationPopover, newMatchDialog, userProfile, geoLocation) {
      if (window.StatusBar) StatusBar.styleLightContent();
      Kismet.collections.matchesData = matchesData;

      var delegate = $ionicScrollDelegate.$getByHandle('matchScroll'),
        waitBeforeNextLoad = false,
        PAUSE_ONEMPTY_SEC = 60,
        PAUSE_LOAD_SEC = 5,
        ionScrollEl = document.getElementsByTagName('ion-scroll')[0],
        currentIndex = currentIndexSaved || 0;

      currentIndexSaved = 0;

      $scope.contentHeight = ionScrollEl.offsetHeight;
      $scope.contentWidth = ionScrollEl.offsetWidth;
      $scope.Math = window.Math;
      $scope.drawerIsLoading = true;
      $scope.error = '';
      $scope.measureDistances = 'km';
      $scope.measureDistancesCalculation = 1;
      $scope.scroll = {
        top: 0
      };
      $scope.undo = matchesData.undoObj;

      init();

      setupDrawer($scope);

      setupListeners($scope);

      settingsData.get().then(function(data){
        $scope.measureDistances = data.preferredDistanceUnit;
        if (data.preferredDistanceUnit != 'km') {
          $scope.measureDistances = 'MEASURE_MILES';
          $scope.measureDistancesCalculation = 0.621371192;
        } else {
          $scope.measureDistances = 'MEASURE_KM';
        }
      });

      notificationPopover.listenAndPopup($scope);

      $scope.like = function(e, match) {
        /*
        Testing matching page

        newMatchDialog.show({
          participants: [
            {imageId: '86f43e28-9c96-47e6-8c05-02f1a8dddff7', firstName: 'bob', profileId: 'something', chatId: 'something'}
          ],
          onClose: function(animate){
            //if (animate) {
            //  // drawer animation here
            //  match.liked = true;
            //  $timeout(function(){
            //    removeItem($scope.matches, match);
            //  }, 500);
            //  chatsCollection.clear();
            //  chatsCollection.load();
            //} else {
            //  removeItem($scope.matches, match);
            //}
          }
        });
        return;*/
        matchesData.like(match).then(function(data){
          if (data.isMatch) {
            console.log('Immediate match.');
            $scope.undo.user = null;

            removeItem($scope.matches, match);
            chatsCollection.clear();
            chatsCollection.load();

            //$rootScope.$broadcast(config.event.NEW_MATCH, {
            newMatchDialog.show({
              participants: [
                {imageId: match.img, firstName: match.firstName, profileId: match.profileId, chatId: data.chatId}
              ],
              onClose: function(animate){
                //if (animate) {
                //  // drawer animation here
                //  match.liked = true;
                //  $timeout(function(){
                //    removeItem($scope.matches, match);
                //  }, 500);
                //  chatsCollection.clear();
                //  chatsCollection.load();
                //} else {
                //  removeItem($scope.matches, match);
                //}
              }
            });
          } else {
            match.liked = true;
            $timeout(function(){
              removeItem($scope.matches, match);
              addToUndo(match);
            }, 500);
          }
        });
      };
      $scope.dislike = function(e, match){
        match.disliked = true;
        matchesData.dislike(match);
        $timeout(function(){
          removeItem($scope.matches, match);
          addToUndo(match);
        }, 500);
      };
      $scope.doRefresh = function(){
        $scope.isLoading = true;
        $scope.undo.user = null;
        if (locationError) {
          checkLocation().finally(function(){ $scope.isLoading = false; });
          return;
        }
        matchesData.get(pageSize, {refresh: true})
          .then(function(matches){
            $scope.matches = matches.concat({type: 'loading'});
            $ionicScrollDelegate.scrollTop(true);
            $scope.isLoading = false;
          })
          .catch(function(error){
            $scope.error = JSON.stringify(error, 1);
            $scope.isLoading = false;
          });
      };
      $scope.gotoProfile = function(profileId) {
        currentIndexSaved = currentIndex;
        $scope.goto('/app/profiles/' + profileId)
      };

      function init() {
        firstInit = false;
        locationError = false;

        var matchesResult = matchesData.get(pageSize, {tryCache: true});
        console.log('init: matchesResult.length=' + matchesResult.length);

        if (matchesResult.concat) {
          matchesResult.length === 0 && matchesResult.push({type: 'loading'});
          $scope.matches = matchesResult;
          console.log('init: loaded matches from cache');
          delegate.scrollToRememberedPosition();
        } else {
          $scope.isLoading = true;
          $scope.matches = [{type: 'loading'}];
          matchesResult.then(function(matches){
              $scope.isLoading = false;
              $scope.matches = matches.concat({type: 'loading'});
              delegate.scrollToRememberedPosition();
            })
            .catch(function(error){
              $scope.error = JSON.stringify(error, 1);
              $scope.isLoading = false;
              if (error.errorCode === 'E004') {
                onLocationError();
              }
            });
        }
        console.log('3 init after getting items ' + ($scope.matches && $scope.matches.length));
      }
      function checkLocation() {
        $scope.matches = [{type: 'loading'}];
        return geoLocation.get().then(geoLocation.send).then(init).catch(onLocationError);
      }
      function onLocationError() {
        locationError = true;
        $scope.matches = [{type: 'location-error'}];
        $scope.isLoading = false;
      }
      function loadMore() {
        $scope.isLoading = true;
        console.log('loading more...');
        waitBeforeNextLoad = true;
        var waitFlagEvaluated = false;
        matchesData.next($scope.matches.length - 1, pageSize).then(function(newItems){
          console.log('loaded new items ' + newItems.length);
          addItems(newItems);
          waitFlagEvaluated = true;
          $timeout(function(){
            waitBeforeNextLoad = false;
          }, (newItems.length === 0 && PAUSE_ONEMPTY_SEC || PAUSE_LOAD_SEC) * 1000);
        }).finally(function(){
          $scope.isLoading = false;
          !waitFlagEvaluated && $timeout(function(){
            waitBeforeNextLoad = false;
          }, PAUSE_LOAD_SEC * 1000);
        });
      }
      function addItems(newItems) {
        var profileIds = _.map(_.prop('profileId'))($scope.matches);
        newItems && ($scope.matches = $scope.matches
          .filter(function(m){ return !!m.profileId;})
          .concat(newItems.filter(function(i){ return profileIds.indexOf(i.profileId) === -1;}))
        );
        $scope.matches.push({type: 'loading'});
      }
      function removeItem(array, item){
        if (!item) return;
        matchesData.remove(item);
        var indexOf = array.indexOf(item);
        if (indexOf !== -1) {
          return array.splice(indexOf, 1);
        }
        return item;
      }
      function getMatchById(array, profileId) {
        return _.find(_.where({profileId: profileId}))(array);
      }
      function addToUndo(item) {
        $scope.undo.user = item;
        item && item.img && $scope.undo.control && $scope.undo.control.refresh && $scope.undo.control.refresh(item.img);
      }
      function undo() {
        var item = $scope.undo.user;
        console.log('undo: currentIndex=' + currentIndex + ', matches=' + $scope.matches.length);
        if (item) {
          $scope.undo.processing = true;
          matchesData.undo(item, currentIndex)
            .then(onUndoSuccess)
            .catch(onUndoError)
            .finally(onUndoComplete);
        }
      }
      function onUndoSuccess(item) {
        console.log('undo success: currentIndex=' + currentIndex);
        $scope.matches.splice(currentIndex, 0, item);
        $timeout(function(){
          $scope.undo.user = null;
        }, config.ttl.MATCH_UNDO_SEC * 800);
      }
      function onUndoError(error) {
        console.log('Undo error: ' + error, error);
        if (error && error.statusCode === 400) {
          $scope.undo.user = null;
          $rootScope.$broadcast('app-error', {text: 'ERROR_MATCH_UNDO_FAILED', timeout: 5000});
        }
      }
      function onUndoComplete() {
        $timeout(function(){
          $scope.undo.processing = false;
        }, config.ttl.MATCH_UNDO_SEC * 1000);
      }
      function cleanupOnNewMatch(participant) {
        removeItem($scope.matches, getMatchById($scope.matches, participant.profileId));
      }
      function setupListeners($scope) {
        $scope.$on('scroll-page-complete', function(event, data) {
          currentIndex = data.scrollTop / data.clientHeight;
          if (!waitBeforeNextLoad && $scope.matches.length - 1 - currentIndex <= 5){
            console.log('Page scroll completed. currentIndex + 1 = ' + (currentIndex + 1) + ' of ' + $scope.matches.length);
            loadMore();
          }
        });
        $scope.$on(config.event.NEW_MATCH, function(event, data){
          cleanupOnNewMatch(data.participants && data.participants[0]);
        });
        $scope.$on('app-resize', function(){
          $scope.$apply(function(){
            $scope.contentHeight = document.getElementsByTagName('ion-scroll')[0].offsetHeight;
            $scope.contentWidth = document.getElementsByTagName('ion-scroll')[0].offsetWidth;
          });
        });
        $scope.$on('undo-match', undo);
        $scope.$on('$destroy', function(){
          matchesData.saveCache($scope.matches);
        });
      }
      function setupDrawer($scope) {
        $scope.drawerList = fillDrawer(chatsCollection.take());
        $scope.drawerIsLoading = false;
        chatsCollection.onUpdate($scope, function(items){
          $scope.drawerList = fillDrawer(chatsCollection.take());
          $scope.drawerIsLoading = false;
        });
      }
    };

    return MatchesCtrl;
  });

}(define));