(function (define) {
  'use strict';

  define(['config'], function (config) {

    var AudioCtrl = function($scope, $rootScope, $q, $ionicPopup, $timeout, mediaUpload){
      var platform =  window.device && device.platform || 'android',
        fileName = "myrecording." + (platform === 'iOS' ? 'wav' : 'mp3'),
        fileDeferred = mediaUpload.getFile(fileName),
        src,
        recordDeferred, mediaRec, playDeferred, mediaPlay;

      
      this.recordStart = function() {
        recordDeferred = $q.defer();

        fileDeferred.then(function(fileEntry){
          //console.log('fileEntry:', fileEntry);
          src = platform === 'iOS' ? fileEntry.fullPath : fileEntry.nativeURL;

          mediaRec = new Media(
            src,
            function(data) {
              recordDeferred.resolve(fileEntry);
            }, function(err) {
              alert("recordAudio():Audio Error: "+ err.code + ', ' + JSON.stringify(err, 1).substring(0,200));
              recordDeferred.reject(err.message);
            });
          mediaRec.startRecord();

          $timeout(function() {
            mediaRec.stopRecord();
            navigator.notification.beep(1);
            navigator.notification.vibrate(1000);
            var alertPopup = $ionicPopup.alert({
              title: 'Recording Limit',
              template: 'Recording limit has been reached'
            });
            alertPopup.then(function(res) {
              //console.log('Saved');
            });


          }, config.media.audioRecordLimit * 1000);

        });

        return recordDeferred.promise;
      };
      this.recordStop = function() {
        mediaRec.stopRecord();
      };

      this.play = function() {
        playDeferred = $q.defer();
        mediaPlay = new Media(src, function() {
            $timeout(function(){
              playDeferred.resolve('done playing');
            }, mediaPlay.getDuration());
          },
          function(err) {
            playDeferred.reject(err.code);
          });
        mediaPlay.play();
        return playDeferred.promise;
      };
      this.stop = function() {
        mediaPlay.stop();
        playDeferred.resolve('playing was stopped');
      };

      this.upload = function(fileEntry, options) {
        //console.log('AudioCtrl.upload: fileEntry:', fileEntry);
        return mediaUpload.uploadFile({
          fullPath: fileEntry.nativeURL,
          name: fileEntry.name,
          size: fileEntry.size,
          type: (platform === 'iOS' ? 'audio/wav' : 'audio/mpeg3'), //mime-type <'audio/mpeg3' | 'audio/wav'>  . fileEntry.type
          mediaType: 'audio'
        }, options);
      };

    };

    return AudioCtrl;
  });

}(define));