
(function (define) {
  'use strict';

  define(['config'], function (config) {

    var mediaUpload = function ($q, userProfile, security) {

      function uploadFile(mediaFile, options) {
        if (!window.FileTransfer){
          return $q.reject();
        }

        var deferred = $q.defer(),
          ft = new FileTransfer(),
          path = mediaFile.fullPath,
          name = mediaFile.name,
          mediaType = mediaFile.mediaType || 'audio',
          uploadUrl = config.server.url + config.apiUrls.UPLOAD_MEDIA
            .replace('{profileId}', userProfile.getProfileId())
            .replace('{mediaType}', mediaType)
            .replace('{mediaId}', options.mediaId)
            .replace('{chatId}', options.chatId);

        //alert('Start uploading file ' + name + ', ' + path + ' to ' + uploadUrl);
        //alert('Uploading file: ' + JSON.stringify(mediaFile, 1));
        //debug.toConsole('Start uploading file ' + name + ', ' + path, mediaFile);
        //console.log('mediaUpload.uploadFile: name='+ name + ', path=' + path, mediaFile);
        ft.upload(
          path,
          uploadUrl,
          //'http://cordova-filetransfer.jitsu.com',
          function(result) {
            var parsed = JSON.parse(result.response);
            var obj = {
              id: parsed.mediaId,
              type: mediaType,
              src: config.getMediaUrl(parsed.mediaId),
              size: result.bytesSent,
              duration: parsed.duration,
              fileName: parsed.filename,
              contentType: (mediaFile.type || parsed.headers['content-type'])
            };
            //alert('Parsed: ' + JSON.stringify(obj, 1));
            //console.log('Upload response parsed: ', parsed);

            deferred.resolve(obj);
          },
          function(error) {
            //alert('Error uploading file ' + path + ': ' + JSON.stringify(error, 1) );
            deferred.reject('Error uploading file');
          },
          { fileName: name, mimeType: mediaFile.type, headers: security.header(uploadUrl, 'POST') });

        return deferred.promise;
      }

      /* Cordova File API */
      function listFiles() {
        window.requestFileSystem(LocalFileSystem.TEMPORARY, 0, gotFS, function(error){  console.log('Failed to get filesystem: ', error); });

        function gotFS(fileSystem) {
          var dirEntry = fileSystem.root;
          var directoryReader = dirEntry.createReader();
          directoryReader.readEntries(readEntries, function failReadEntries(error) { console.log("Failed to list directory contents: " + error.code, error);});
        }
        function readEntries(entries) {
          //console.log('readEntries:');
          for (var i=0; i<entries.length; i++) {
            //console.log('- ' + entries[i].name);
          }
        }
      }
      function getFile(fileName) {
        if (!window.LocalFileSystem){
          return $q.reject();
        }

        var fileDeferred = $q.defer();

        window.requestFileSystem(LocalFileSystem.TEMPORARY, 0, gotFS, function(error){  console.log('Failed to get filesystem: ', error); });

        function gotFS(fileSystem) {
          fileSystem.root.getFile(fileName, {create: true},
            function(fileEntry){
              fileDeferred.resolve(fileEntry);
            },
            function(error) {
              //console.log('Failed to get file ' + fileName, error);
              fileDeferred.reject(error);
            }
          );
        }
        return fileDeferred.promise;
      }

      return {
        uploadFile: uploadFile,
        getFile: getFile,
        listFiles: listFiles
      }
    };

    return mediaUpload;
  });
}(define));