(function (define) {
  'use strict';

  define(['config'], function (config) {

    var appEvents = function ($rootScope, $location, $q, $translate, newMatchDialog, userProfile, $ionicPopup, chatsCollection, notificationPopover) {

      var state = {
        isOffline: function(){
          if (navigator.network) {
            return navigator.network.connection.type === Connection.NONE;
          } else {
            return false;
          }
        },
        isBackground: false
      };

      window.$rootScope = $rootScope;
      $rootScope.$on(config.event.NEW_MATCH, function(event, data){
        var participant = data.participants && data.participants[0];
        notificationPopover.show({
          imageId: participant && participant.imageId,
          message: 'New match! Say hello to ' + (participant && participant.firstName),
          action: function(){
            participant && $location.path('/app/chat-room/' + participant.chatId);
            notificationPopover.hide();
          }
        });
        //newMatchDialog.show(data);
        chatsCollection.clear();
        chatsCollection.load();
      });

      function init() {
        document.addEventListener('offline', function(){
          $rootScope.$broadcast('app-offline');
        });
        document.addEventListener('online', function(){
          $rootScope.$broadcast('app-online');
        });
        window.addEventListener('resize', function(){
          $rootScope.$broadcast('app-resize');
        });
        window.addEventListener('orientationchange', function(){
          $rootScope.$broadcast('app-resize');
        });

        document.addEventListener('resume', function(){
          console.log('AppEvents: EVENT resume');
          state.isBackground = false;
          $rootScope.$broadcast('app-resume');
        });

        document.addEventListener('pause', function(){
          console.log('AppEvents: EVENT pause');
          state.isBackground = true;
          $rootScope.$broadcast('app-pause');
        });

        if (window.cordova) {
        cordova.exec(
          function(){
            state.isBackground = true;
            $rootScope.$broadcast('screenshot-taken');},
          function(){
            console.log("AppEvents: Failed to Start Screenshot plugin.")},
          "ScreenshotPrivacyPlugin",
          "startListener",
          []);
        }

      }

        function get(name){
          var param = state[name];
          return param && (typeof param === 'function' ? state[name]() : param);
        }

        $rootScope.$on('forced-logout', function(){
          if (userProfile.isLoggedIn()) {
            navigator.vibrate(config.popups.confirmVibrateTime);
            $location.path('/app/force-logout');
            $q.all({
              okText: $translate('POPUP_OK'),
              title: $translate('POPUP_FORCED_LOGOUT_TITLE'),
              text: $translate('POPUP_FORCED_LOGOUT_TEXT')
            })
            .then(function (translations) {
              var confirmPopup = $ionicPopup.alert({
                title: translations.title,
                template: translations.text,
                okText: translations.okText
              });
            })
            .catch(function () {
              var confirmPopup = $ionicPopup.confirm({
                title: 'POPUP_FORCED_LOGOUT_TITLE',
                template: 'POPUP_FORCED_LOGOUT_TEXT',
                okText: 'POPUP_OK'
              });
            });
          } else {
            $location.path('/app/force-logout');
          }
        });

        return {
          init: init,
          get: get
        };
      };

      return appEvents;
    })
  }(define));
