
(function (define) {
  'use strict';

  define(['config'], function (config) {

    var appInit = function(
      $ionicPlatform, $location, userProfile, $timeout, pushData, notificationPopover, appEvents, appState,
      security, mqttTransport, localeLang, touchAuth, cropAreaCircle, chatsCollection, geoLocation, settingsData
    ){
      if (!window.Kismet) window.Kismet = {};

      var deviceInfo = {};

      function onStart () {
        window.cropAreaCircle = cropAreaCircle;
        $ionicPlatform.ready(function () {

          appEvents.init();

          if (window.device) {
            deviceInfo.deviceID = device.uuid;
            deviceInfo.userPlatform = device.platform;
            deviceInfo.userPlatformVersion = device.version;
            deviceInfo.userDeviceModel = device.model;
            if( device.model.indexOf('iPhone4') >= 0){
              angular.element(document.querySelector('body')).addClass('device-iPhone4');
            }
          }

          // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
          // for form inputs)
          if (window.cordova && window.cordova.plugins.Keyboard) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            cordova.plugins.Keyboard.disableScroll(true);
          }

          if (window.analytics) {
            analytics.startTrackerWithId('UA-53821839-2');
            // TODO: have debug Flag and use here
            if (false) analytics.debugMode();
          }

          localeLang.init();

          notificationPopover.setup();

          if(navigator.splashscreen){
            $timeout(function() {
              navigator.splashscreen.hide();
            }, config.splash.delay);
          }


//        touchAuth.isAuthenticated()
//          .then(function(data){
//            alert('Touch auth OK with: ' + data);
//          })
//          .catch(function(error){
//            alert('Cant touch auth ' + JSON.stringify(error));
//          })

        });
        $rootScope.$on('logged-in', function(event, data){ onLogin(data); });
        $rootScope.$on('profile-ready', function(event, data){ onProfileReady(data); });
        $rootScope.$on('logout', function(event, data){ onLogout(data); });
        $rootScope.$on('app-online', registerDevice);
      }
      function onLogin(loginInfo) {
        console.log('------- onLogin ' + loginInfo.profileId, loginInfo);
        security.setup(loginInfo.profileId, loginInfo.key);
        geoLocation.init();

        // REGISTER DEVICE:
        registerDevice();
      }
      function onProfileReady(profileInfo) {
        console.log('------- onProfileReady');
        chatsCollection.load();
        mqttTransport.init();
      }
      function onLogout() {
        console.log('------- onLogout');
        security.reset();
        geoLocation.stop();
        settingsData.reset();
        appState.set('isDeviceRegistered', false);
      }
      function registerDevice() {
        if (deviceInfo.deviceID && userProfile.isLoggedIn() && !appState.get('isDeviceRegistered')) {
          localeLang.ready()
            .then(function(localeObj){
              console.log('----- onLogin: registerDevice');
              return userProfile.registerDevice(deviceInfo, localeObj)
            })
            .then(function(){
              appState.set('isDeviceRegistered', true);
              pushData.init(deviceInfo);
            }, function(error){
              console.log('ERROR: could not register device. Will log out... Error message: ' + JSON.stringify(error));
              //$location.path('/app/force-logout');
            });
        } else {
          console.log('[registerDevice]: noop. isDeviceRegistered=' + appState.get('isDeviceRegistered'));
        }
      }

      return {
        onStart: onStart,
        onLogin: onLogin,
        onLogout: onLogout
      }
    };

    return appInit;
  })
}(define));
