
(function (define) {
  'use strict';

  define(['config'], function (config) {

    var geoLocation = function ($rootScope, $q, baseData, userProfile, $interval) {

      var errorCounter = 0,
        intervalPromise;

      function getLocation() {
        var geoDeferred = $q.defer();

        if (navigator.geolocation) {
          navigator.geolocation.getCurrentPosition(
            function(pos) {
              var locationData = {
                'latitude': pos.coords.latitude,
                'longitude': pos.coords.longitude,
                'accuracy': pos.coords.accuracy
              };

              geoDeferred.resolve(locationData);
            },
            function(error){
              geoDeferred.reject("geolocation Error: " + error);
            },
            { maximumAge: 3000, timeout: 5000, enableHighAccuracy: true }
          );
        } else {
          geoDeferred.reject('not available');
        }

        return geoDeferred.promise;
      }

      function timerLocation() {
        $rootScope.$broadcast('app-location-good');
        return getLocation()
          .then(sendLocation)
          .catch(onLocationErr);
      }

      function sendLocation (locationData) {
        var profileId = userProfile.getProfileId();
        $rootScope.$broadcast('app-location-good');
        if (profileId) {
          return baseData.post('/api/profile/location?profileId={profileId}', locationData, profileId);
        }
        return $q.reject('Cannot send location: No profile ID');
      }

      function onLocationErr(error) {
        errorCounter++;
        if (errorCounter > 1) {
          errorCounter = 0;
          $rootScope.$broadcast('app-location-error');
        }
      }

      function init() {
        //console.log('getLocation init...');
        intervalPromise = $interval(timerLocation, config.geoLocationRefresh.REFRESH_TIME*1000);
        return timerLocation();
      }

      function stop() {
        $interval.cancel(intervalPromise);
      }


      return {
        get: getLocation,
        init: init,
        stop: stop,
        send: sendLocation
      };
    };

    return geoLocation;
  });
}(define));