
(function (define) {
  'use strict';

  define(['config'], function (config) {

    var localeLang = function ($q, $translate, amMoment) {

      var deferred = $q.defer(),
        langDeferred = $q.defer(),
        localeDeferred = $q.defer();

      function init() {
        return detectLanguage()
          .then(function(localeLang){
            var preferredLanguage = localeLang.lang;
            var preferredLanguageTwoLetter = localeLang.lang.substring(0, 2);

            if (config.lang.localeCodes.indexOf(preferredLanguage) != -1) {
              $translate.use(preferredLanguage);
              amMoment.changeLocale(preferredLanguage);
            } else if (config.lang.localeCodes.indexOf(preferredLanguageTwoLetter+'-'+localeLang.locale) != -1) {
              $translate.use(preferredLanguageTwoLetter+'-'+localeLang.locale);
              amMoment.changeLocale(preferredLanguageTwoLetter+'-'+localeLang.locale);
            } else if (config.lang.localeCodes.indexOf(preferredLanguageTwoLetter+'-'+config.lang.defaultLocale) != -1) {
              $translate.use(preferredLanguageTwoLetter+'-'+config.lang.defaultLocale);
              amMoment.changeLocale(preferredLanguageTwoLetter+'-'+config.lang.defaultLocale);
            } else if (config.lang.localeCodes.indexOf(preferredLanguageTwoLetter) != -1) {
              $translate.use(preferredLanguageTwoLetter);
              amMoment.changeLocale(preferredLanguageTwoLetter);
            } else {
              $translate.use(config.lang.defaultLocale);
              amMoment.changeLocale(config.lang.defaultLocale);
            }
            return localeLang;
          })
          .catch(function(){
            $translate.use(config.lang.defaultLocale);
            amMoment.changeLocale(config.lang.defaultLocale);
          })
      }

      function detectLanguage() {
        if (navigator.globalization) {
          navigator.globalization.getPreferredLanguage(
            function(language) {
              langDeferred.resolve(language.value);
            },
            function(error) {
              langDeferred.reject('Cant get language. ' + error);
            }
          );
          navigator.globalization.getLocaleName(
            function(local) {
              localeDeferred.resolve(local.value);
            },
            function(error){
              localeDeferred.reject('Error: navigator.globalization.getLocaleName cannot retrieve locale. ' + error);
            }
          );
          $q.all({lang: langDeferred.promise, locale: localeDeferred.promise})
            .then(function(data){
              deferred.resolve(data);
            });
        } else {
          deferred.resolve({
            lang: 'en-ca',
            locale: 'ca'
          });
        }
        return deferred.promise;
      }

      return {
        init: init,
        ready: function(){ return deferred.promise; }
      }
    };

    return localeLang;
  });
}(define));