
(function (define) {
  'use strict';

  define(['config'], function (config) {

    var security = function () {
      var self = this;


      this.credentials = {
        id: null,
        key: null,
        algorithm: config.security.algorithm
      };



      function setCredentials(profileId, key) {
        self.credentials.id = profileId;
        self.credentials.key = key;
        //console.log('[hawk.setCredentials]: credentials', self.credentials);

        this.credentials = {
          id: profileId,
          key: key,
          algorithm: config.security.algorithm
        };
      }

      function reset(){
        self.credentials.id = null;
        self.credentials.key = null;
        //console.log('[hawk.reset]: reset credentials.');
      }

      function getSignature() {
        var ts = hawk.utils.now(),
          nonce = hawk.utils.randomString(6),
          mac = hawk.crypto.calculateTsMac(ts, self.credentials),
          signature = hawk.utils.base64urlEncode(self.credentials.id + '\\' + ts + '\\' + nonce + '\\' + mac);

        return signature;
      }

      //var header = hawk.client.header(url, 'GET', { credentials: credentials });
      //headers: { Authorization: header.field }
      function getHeader(url, method) {
        if (!self.credentials.id){
          return {};
        }

        var header;
        method = method || 'GET';
        try {
          header = hawk.client.header(url, method, { credentials: self.credentials });
        } catch(e) {
          //TODO: report the error
          header = {field: 'unknown'};
        }
        return { Authorization: header.field };
      }

      //var bewit = hawk.client.bewit(url, { credentials: credentials, ttlSec: 10 });
      function getBewit(url, ttl) {
        if (!self.credentials.id){
          return '';
        }
        var bewit;
        ttl = typeof ttl === 'undefined' ? 10 : ttl;
        try {
          bewit = hawk.client.bewit(url, { credentials: self.credentials, ttlSec: ttl });
        } catch(e) {
          //TODO: report the error
          bewit = 'unknown';
        }
        return '&bewit=' + bewit;
      }


      return {
        setup: setCredentials,
        reset: reset,
        bewit: getBewit,
        header: getHeader,
        getSignature: getSignature
      };
    };

    return security;
  });
}(define));
